#!/usr/bin/env python
# encoding: utf-8
"""
tools.py - Functions for date manipulation

Created by Christian Klein on 2010-04-22.
Copyright (c) 2010 HUDORA GmbH. All rights reserved.
"""

import datetime
import unittest


def date_trunc(date, trtype):
    """
    Truncate date

    This function is inspired by date_trunc from PostgreSQL, see
    http://www.postgresql.org/docs/8.1/static/functions-datetime.html#FUNCTIONS-DATETIME-TRUNC
    """

    tmp = date.timetuple()
    if trtype == "year":
        return datetime.datetime(tmp.tm_year, 1, 1)
    elif trtype == "month":
        return datetime.datetime(tmp.tm_year, tmp.tm_mon, 1)
    elif trtype == "week":
        firstday = date - datetime.timedelta(days=tmp.tm_wday)
        return datetime.datetime.combine(firstday, datetime.time(0))
    elif trtype == "day":
        return datetime.datetime(tmp.tm_year, tmp.tm_mon, tmp.tm_mday)
    elif trtype == "hour":
        return datetime.datetime(tmp.tm_year, tmp.tm_mon, tmp.tm_mday, tmp.tm_hour)
    elif trtype == "minute":
        return datetime.datetime(tmp.tm_year, tmp.tm_mon, tmp.tm_mday, tmp.tm_hour, tmp.tm_min)
    elif trtype == "second":
        return datetime.datetime(tmp.tm_year, tmp.tm_mon, tmp.tm_mday, tmp.tm_hour, tmp.tm_min, tmp.tm_sec)
    else:
        raise ValueError("Unknown ")


def get_week(date):
    """
    Calculates the week of the year for a given date
    and returns the year and week number.
    """

    date = date_trunc(date, 'week')

    first_monday = date_trunc(date_trunc(date, 'year'), 'week')
    if first_monday.year < date.year:
        first_monday += datetime.timedelta(days=7)
    diff = date_trunc(date, 'day') - first_monday
    week = 1 + (diff.days / 7)
    return week, first_monday.year


class DateTruncTestCase(unittest.TestCase):
    """Unittests for date_trunc"""

    def test_truncate_year(self):
        self.assertEqual(date_trunc(datetime.datetime(1980, 5, 4), 'year'), datetime.datetime(1980, 1, 1))
        self.assertEqual(date_trunc(datetime.datetime(1980, 5, 4), 'year').date(), datetime.date(1980, 1, 1))
        self.assertEqual(date_trunc(datetime.date(1980, 5, 4), 'year'),
                         datetime.datetime(1980, 1, 1, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(1980, 5, 4), 'year').date(), datetime.date(1980, 1, 1))

    def test_truncate_month(self):
        self.assertEqual(date_trunc(datetime.datetime(1978, 6, 12), 'month'), datetime.datetime(1978, 6, 1))
        self.assertEqual(date_trunc(datetime.datetime(1978, 6, 12), 'month').date(),
                         datetime.date(1978, 6, 1))
        self.assertEqual(date_trunc(datetime.date(1978, 6, 12), 'month'),
                         datetime.datetime(1978, 6, 1, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(1978, 6, 12), 'month').date(), datetime.date(1978, 6, 1))

    def test_truncate_week(self):
        self.assertEqual(date_trunc(datetime.datetime(2000, 1, 1), 'week'), datetime.datetime(1999, 12, 27))
        self.assertEqual(date_trunc(datetime.datetime(2000, 1, 1), 'week').date(),
                         datetime.date(1999, 12, 27))
        self.assertEqual(date_trunc(datetime.date(2000, 1, 1), 'week'),
                         datetime.datetime(1999, 12, 27, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(2000, 1, 1), 'week').date(), datetime.date(1999, 12, 27))

    def test_truncate_day(self):
        self.assertEqual(date_trunc(datetime.datetime(2006, 2, 25, 23, 17, 40), 'day'),
                         datetime.datetime(2006, 2, 25))
        self.assertEqual(date_trunc(datetime.datetime(2006, 2, 25), 'day'), datetime.datetime(2006, 2, 25))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'day'),
                         datetime.datetime(2006, 2, 25, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'day').date(), datetime.date(2006, 2, 25))

    def test_truncate_hour(self):
        self.assertEqual(date_trunc(datetime.datetime(2006, 2, 25, 23, 17, 40), 'hour'),
                         datetime.datetime(2006, 2, 25, 23))
        self.assertEqual(date_trunc(datetime.datetime(2006, 2, 25, 23, 17, 40), 'hour'),
                         datetime.datetime(2006, 2, 25, 23, 0, 0))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'hour'),
                         datetime.datetime(2006, 2, 25, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'hour').date(), datetime.date(2006, 2, 25))

    def test_truncate_minute(self):
        self.assertEqual(date_trunc(datetime.datetime(2006, 2, 25, 23, 17, 40), 'minute'),
                         datetime.datetime(2006, 2, 25, 23, 17, 0))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'minute'),
                         datetime.datetime(2006, 2, 25, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'minute').date(),
                         datetime.date(2006, 2, 25))

    def test_truncate_second(self):
        self.assertEqual(date_trunc(datetime.datetime(2006, 2, 25, 23, 17, 40), 'second'),
                         datetime.datetime(2006, 2, 25, 23, 17, 40))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'second'),
                         datetime.datetime(2006, 2, 25, 0, 0, 0))
        self.assertEqual(date_trunc(datetime.date(2006, 2, 25), 'second').date(),
                         datetime.date(2006, 2, 25))

    def test_invalid(self):
        self.assertRaises(ValueError, date_trunc, datetime.datetime.now(), 'alex')
        self.assertRaises(AttributeError, date_trunc, 'alex', 'day')


class WeekTestCase(unittest.TestCase):
    """Unittests for get_week"""
    def test_week(self):
        self.assertEqual(get_week(datetime.datetime(1979, 1, 1)), (1, 1979))
        self.assertEqual(get_week(datetime.datetime(1980, 1, 1)), (53, 1979))
        self.assertEqual(get_week(datetime.datetime(1980, 1, 2)), (53, 1979))
        self.assertEqual(get_week(datetime.datetime(1980, 1, 6)), (53, 1979))
        self.assertEqual(get_week(datetime.datetime(1980, 1, 7)), (1, 1980))
        self.assertEqual(get_week(datetime.datetime(1980, 5, 4)), (17, 1980))
        self.assertEqual(get_week(datetime.datetime(1990, 1, 1)), (1, 1990))
        self.assertEqual(get_week(datetime.datetime(1989, 12, 31)), (52, 1989))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
hutools/obfuscation.py this contains code to hide data from your little sister.

THIS IS NOT INDUSTRIAL-STRENGTH CRYPTO - use OpenSSL abd friends if you need real crypto!

Created by Maximillian Dornseif on 2009-08-31.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

import sys
import random


class RC4:
    """RC4 en-/decription.

    Based on code from Thomas Dixon at http://www.macshadows.com/forums/index.php?showtopic=5766

    >>> RC4('sekrit').decrypt(RC4('sekrit').encrypt('foo'))
    'foo'
    """
    def __init__(self, key):
        """initialize the state table."""
        random.seed()
        (self.x, self.y) = (0, 0)
        self.state_array = [i for i in range(0, 256)]
        for i in range(0, 256):
            self.x = ((ord(key[i % len(key)]) & 0xff) + self.state_array[i] + self.x) & 0xff
            self.state_array[i], self.state_array[self.x] = self.state_array[self.x], self.state_array[i]
        self.x = 0

    def engine_crypt(self, input):
        self.out = []
        for i in range(0, len(input)):
            self.x = (self.x + 1) & 0xff
            self.y = (self.state_array[self.x] + self.y) & 0xff
            self.state_array[self.x], self.state_array[self.y] = \
                self.state_array[self.y], self.state_array[self.x]
            self.out.append(chr((ord(input[i]) ^
                                 self.state_array[(self.state_array[self.x]
                                                   + self.state_array[self.y]) & 0xff])))
        return "".join(self.out)

    def encrypt(self, cleartext, iv=None):
        if not iv:
            iv = chr(random.randint(0, 255)) + chr(random.randint(0, 255))
        return self.engine_crypt(str(iv)[-2:] + str(cleartext))

    def decrypt(self, ciphertext):
        ret = self.engine_crypt(ciphertext)
        return ret[2:]

if __name__ == "__main__":
    import doctest
    failure_count, test_count = doctest.testmod()
    sys.exit(failure_count)



#!/usr/bin/python

"""Module for Communication via NetStrings.

This module holds a single class named NetStringIO which wraps
about a file handle and uses NetStrings vor all Communication via
this filehandle.  For more information in netstrings see
http://cr.yp.to/proto/netstrings.txt


Authors:

This Software was written by drt@un.bewaff.net for the twisd AG,
Bonn, Germany. The twisd AG kindly donated it as Freie Software.


"""

__version__ = '$Id: NetStringIO.py,v 1.5 2001/05/03 22:05:57 drt Exp$'

__copyright__ = """(c) 2001 twisd AG, Bonn - http://www.twisd.de/
further distribution is granted under the terms of LGPL or classical
MIT Licence."""

# * Im not sure at the moment what NetStringIO has to implement to
#   qualify as an file-like object.
#
# * Should I define an own error object for this module instead of using
#   IOerror?
#
# * Should I implement readline(), readlines() and writelinesU()?
#   They do not realy make sense withe netstrings, do they?

import string


class NetStringIO:
    """Warping arround a file object changing all I/O to netstrings.

    I use this class to warp it arround filehandles obtained by a call
    to socket.makefile(). With this I can build protocols using
    netstrings with ease. For more information in netstrings see
    http://cr.yp.to/proto/netstrings.txt

    I like sending an '\n' after every netstring. This makes
    inspection of data on the wire much more readable. You can change
    this behaviour by calling the constructor with an optional empty
    string argument. You even can specify another delimiter than '\n'
    """

    def __init__(self, fileo, delim='\n'):
        """Create a Netstring object warping a file-like object.

        n = NetStringIO.NetStringIO(file [, delimiter])

        The returned object should work like reading or writing
        directly from file while the difference that all reading or
        writing is preformed via netstrings. If you leave out
        'delimiter' every netstring will be followed by '\n' while
        writing and is expected to be followed by '\n' while
        reading. By passing a second parameter to the constructor you
        can change '\n' to something else. By passing '' as second
        parameter you can get the behaviour which is used in bernstein
        protocols: no delimiter between netstrings.
        """

        self.file = fileo
        self.delim = delim

    def close(self):
        "Closes the underlying file object."

        self.file.close()

    def isatty(self):
        """Checks if the underlying file Object is a tty.

        I guess using ttys with netstrings is of limited use but who
        knows.
        """

        return self.file.isatty()

    def read(self):
        """Read from the underlying file object.

        If the data does not contain a valid netstring or is not
        followed by the delimiter we raise IOerror.

        This is more or less an 1:1 port of the code in
        http://cr.yp.to/proto/netstrings.txt - any more pythonish ways
        to do this?
        """

        # read length of netstring
        l = ''
        c = self.file.read(1)
        while c:
            if c == ':':
                # we are ready reading the length
                break
            elif not c:
                raise IOError("short netstring read at length specification")
            elif c not in string.digits:
                raise IOError("not a valid netstring: %s is not digit" % c)
            l += c
            c = self.file.read(1)

        if not c:
            return None

        strlen = int(l)

        # reading from a socket can take multiple attempts . see
        # any text about socket programming - XXX but doesn't
        # handle read this already for us? To check!
        s = ""
        while len(s) < strlen:
            s1 = self.file.read(strlen - len(s))
            if s1 == "":
                raise IOError("short netstring read at netstring body")
            s = s + s1

        c = self.file.read(1 + len(self.delim))
        if c != "," + self.delim:
            raise IOError("not a valid netstring: wrong termination")

        return s

    def write(self, s):
        """Writes a Netstring to the underlying fileobject."""

        # TODO: check if we have to check for short writes
        self.file.write("%lu:%s,%s" % (len(s), s, self.delim))
        self.file.flush()


def test():
    """A little test suite."""

    import StringIO

    testtext = 'Netstrings rule'
    inf = StringIO.StringIO()
    outf = StringIO.StringIO()

    print "Writing a Netstring ... ",
    f = NetStringIO(outf)
    f.write(testtext)
    print outf.getvalue(),

    inf = StringIO.StringIO(outf.getvalue())
    f.close()

    print "Reading this Netstring ... ",

    fz = NetStringIO(inf)
    ret = fz.read()
    assert ret == testtext, "String is different after reading"
    print ret
    fz.close()

if __name__ == '__main__':
    test()



# -*- Import: -*-
from paver.easy import *
from paver.setuputils import setup
from setuptools import find_packages

try:
    # Optional tasks, only needed for development
    # -*- Optional import: -*-
    from github.tools.task import *
    import paver.doctools
    import paver.virtual
    import paver.misctasks
    ALL_TASKS_LOADED = True
except ImportError, e:
    info("some tasks could not not be imported.")
    debug(str(e))
    ALL_TASKS_LOADED = False



version = '0.61'

classifiers = [
    # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    "Development Status :: 5 - Production/Stable",
    ]

install_requires = [
    # -*- Install requires: -*-
    'setuptools',
    'decorator'
    ]

entry_points="""
    # -*- Entry points: -*-
    """

# compatible with distutils of python 2.3+ or later
setup(
    name='huTools',
    version=version,
    description='Various tiny tools and toys to make Python coding less work more fun.',
    long_description=open('README.rst', 'r').read(),
    classifiers=classifiers,
    keywords='python library',
    author='Maximillian Dornseif',
    author_email='md@hudora.de',
    url='http://hudora.github.com/huTools/',
    license='BSD',
    packages = find_packages(exclude=['bootstrap', 'pavement',]),
    include_package_data=True,
    test_suite='nose.collector',
    zip_safe=False,
    install_requires=install_requires,
    entry_points=entry_points,
    )

options(
    # -*- Paver options: -*-
    minilib=Bunch(
        extra_files=[
            # -*- Minilib extra files: -*-
            ]
        ),
    sphinx=Bunch(
        docroot='docs',
        builddir="_build",
        sourcedir=""
        ),
    virtualenv=Bunch(
        packages_to_install=[
            # -*- Virtualenv packages to install: -*-
            'github-tools',
            "nose",
            'decorator',
            "Sphinx>=0.6b1",
            "pkginfo", 
            "virtualenv"],
        dest_dir='./virtual-env/',
        install_paver=True,
        script_name='bootstrap.py',
        paver_command_line=None
        ),
    )

options.setup.package_data=paver.setuputils.find_package_data(
    'huTools', package='huTools', only_in_packages=False)

if ALL_TASKS_LOADED:
    @task
    @needs('generate_setup', 'minilib', 'setuptools.command.sdist')
    def sdist():
        """Overrides sdist to make sure that our setup.py is generated."""



#!/usr/bin/env python
# encoding: utf-8
"""
tools.py - various helpers for HTTP access

Created by Maximillian Dornseif on 2010-10-24.
Copyright (c) 2010, 2011 HUDORA. All rights reserved.
"""


# quoting based on
# http://svn.python.org/view/python/branches/release27-maint/Lib/urllib.py?view=markup&pathrev=82940
# by Matt Giuca
always_safe = ('ABCDEFGHIJKLMNOPQRSTUVWXYZ'
               'abcdefghijklmnopqrstuvwxyz'
               '0123456789' '_.-')
_safe_map = {}
for i, c in zip(xrange(256), [chr(x) for x in xrange(256)]):
    _safe_map[c] = c if (i < 128 and c in always_safe) else ('%%%02X' % i)
_safe_quoters = {}


def quote(s, safe='/', encoding=None, errors=None):
    """quote('abc def') -> 'abc%20def'

    Each part of a URL, e.g. the path info, the query, etc., has a
    different set of reserved characters that must be quoted.

    RFC 2396 Uniform Resource Identifiers (URI): Generic Syntax lists
    the following reserved characters.

    reserved    = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" |
                  "$" | ","

    Each of these characters is reserved in some component of a URL,
    but not necessarily in all of them.

    By default, the quote function is intended for quoting the path
    section of a URL.  Thus, it will not encode '/'.  This character
    is reserved, but in typical usage the quote function is being
    called on a path where the existing slash characters are used as
    reserved characters.

    string and safe may be either str or unicode objects.

    The optional encoding and errors parameters specify how to deal with the
    non-ASCII characters, as accepted by the unicode.encode method.
    By default, encoding='utf-8' (characters are encoded with UTF-8), and
    errors='strict' (unsupported characters raise a UnicodeEncodeError).
    """
    # fastpath
    if not s:
        return s

    if encoding is not None or isinstance(s, unicode):
        if encoding is None:
            encoding = 'utf-8'
        if errors is None:
            errors = 'strict'
        s = s.encode(encoding, errors)
    if isinstance(safe, unicode):
        # Normalize 'safe' by converting to str and removing non-ASCII chars
        safe = safe.encode('ascii', 'ignore')

    cachekey = (safe, always_safe)
    try:
        (quoter, safe) = _safe_quoters[cachekey]
    except KeyError:
        safe_map = _safe_map.copy()
        safe_map.update([(c, c) for c in safe])
        quoter = safe_map.__getitem__
        safe = always_safe + safe
        _safe_quoters[cachekey] = (quoter, safe)
    if not s.rstrip(safe):
        return s
    return ''.join(map(quoter, s))


def quote_plus(s, safe='', encoding=None, errors=None):
    """Quote the query fragment of a URL; replacing ' ' with '+'"""
    if ' ' in s:
        s = quote(s, safe + ' ', encoding, errors)
        return s.replace(' ', '+')
    return quote(s, safe, encoding, errors)


def urlencode(query):
    """Encode a sequence of two-element tuples or dictionary into a URL query string.

    If the query arg is a sequence of two-element tuples, the order of the
    parameters in the output will match the order of parameters in the
    input.
    """

    if hasattr(query, 'items'):
        # mapping objects
        query = query.items()
    l = []
    for k, v in query:
        k = quote_plus(k)
        if isinstance(v, basestring):
            v = quote_plus(v)
            l.append(k + '=' + v)
        else:
            v = quote_plus(unicode(v))
            l.append(k + '=' + v)
    return '&'.join(l)



# -*- coding: utf-8 -*-

import os

from pkg_resources import parse_version
import pkginfo

def _egg_info(path_to_egg='../'):
    path_to_egg = os.path.join(
        os.path.dirname(__file__), path_to_egg)
    egg_info = pkginfo.Develop(path_to_egg)
    release = egg_info.version
    parsed_version = parse_version(release)
    version = '%s.%s' % tuple([int(x) for x in parsed_version[0:2]])
    return egg_info.name, egg_info.author, version, release

project, author, version, release = _egg_info()
copyright = '2006-2010, Maximillian Dornseif'

# Extension
extensions = [
    # -*-Extensions: -*-
    'sphinx.ext.autodoc'
    ]
intersphinx_mapping = {'http://docs.python.org/': None}

# Source
master_doc = 'index'
templates_path = ['_templates']
source_suffix = '.rst'
exclude_trees = []
pygments_style = 'sphinx'

# html build settings
html_theme = 'default'
html_static_path = ['_static']

# htmlhelp settings
htmlhelp_basename = '%sdoc' % project

# latex build settings
latex_documents = [
    ('index', '%s.tex' % project, u'%s Documentation' % project,
    author, 'manual'),
]



#!/usr/bin/env python
# encoding: utf-8
"""
async.py - functions for asyncrounous code

Created by Maximillian Dornseif on 2009-02-15.
Copyright (c) 2009 HUDORA. All rights reserved.
parts based on http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/84317
"""

import copy
import sys
import threading


class Future:
    """This calls a function in a separate thread and returns a function waiting for that thread to finish.

    Typical use:

    tagfuture = Future(get_slow_tagcount, 'parameter') # starts a separate thread
    ... do something else ...
    tagcount = tagfuture() # waits for thread to finish and returns result

    """

    def __init__(self, func, *args, **kwargs):
        # Constructor
        self.__done = False
        self.__result = None
        self.__status = 'working'
        self.__excpt = None

        self.__Cond = threading.Condition()   # Notify on this Condition when result is ready

        # Run the actual function in a separate thread
        self.__Thread = threading.Thread(target=self.Wrapper, args=((func, ) + args), **kwargs)
        self.__Thread.setName("FutureThread")
        self.__Thread.start()

    def __repr__(self):
        return '<Future at ' + hex(id(self)) + ':' + self.__status + '>'

    def __call__(self):
        self.__Cond.acquire()
        while self.__done == False:
            self.__Cond.wait()
        self.__Cond.release()
        # We deepcopy __result to prevent accidental tampering with it.
        ret = copy.deepcopy(self.__result)
        if self.__excpt:
            raise self.__excpt[0], self.__excpt[1], self.__excpt[2]
        return ret

    def Wrapper(self, func, *args, **kwargs):
        # Run the actual function, and let us housekeep around it
        self.__Cond.acquire()
        try:
            self.__result = func(*args, **kwargs)
        except:
            self.__result = "Exception raised within Future"
            self.__excpt = sys.exc_info()
        self.__done = True
        self.__Cond.notify()
        self.__Cond.release()



#!/usr/bin/env python
# encoding: utf-8
"""
Client library for REST APIs

Created by Christian Klein on 2010-07-21.
Copyright (c) 2010 HUDORA. All rights reserved.
"""

import huTools.http._httplib2 as httplib2
import os
import tempfile
import urllib
import urlparse
import simplejson as json


# TODO: rewrite to use httplib / urllib. httplib2 has issues with Appengine and reliability

def build_url(base, *args):
    """
    Create URLs for a REST API

    >>> build_url('orders_', 1)
    'orders/1/'
    >>> build_url('cool_mc_cool_', 1, 2)
    'cool/1/mc/2/cool/'
    >>> build_url('cool_mc_cool_', 1, 2, 3)
    'cool/1/mc/2/cool/3/'
    """

    tmp = []
    args = list(args)
    components = base.split('_')

    while components:
        tmp.append(components.pop(0))
        if args:
            tmp.append(str(args.pop(0)))
    return os.path.join(*tmp)  # pylint: disable=W0142


class ClientException(Exception):
    """Basisklasse für Exceptions"""
    pass


class ClientNotFoundException(ClientException):
    """ HTTP Status 404 - das Dokument wurde nicht gefunden """
    pass


class ClientForbiddenException(ClientException):
    """ HTTP Status 403 - kein Zugriff erlaubt """
    pass


class ClientUnauthorizedExecption(ClientException):
    """ HTTP Status 401 - Authentifizierung via OAuth/ BasicAuth fehlt """
    pass


class ClientServerErrorExecption(ClientException):
    """ HTTP Status 500 - wenn ein allgemeiner Serverfehler aufgetreten ist """
    pass


class Client(object):
    """
    Client for RESTful API

    All data is encoded as JSON.
    """

    def __init__(self, username, password, endpoint=None):
        self.username = username
        self.password = password
        self.endpoint = endpoint

        # TODO: move to huTools.http.fetch
        cachedir = os.path.join(tempfile.gettempdir(), 'robotrock')
        self.connection = httplib2.Http(cache=cachedir, timeout=20)
        self.connection.add_credentials(username, password, domain=urlparse.urlsplit(self.endpoint).netloc)

    def __getattr__(self, method):
        def handler(*args, **kwargs):
            """doc for handler"""
            return self(method, *args, **kwargs)
        return handler

    def __call__(self, fnc, *args, **kwargs):
        """Do a remote procedure call via HTTP"""

        headers = {'content-type': 'application/json'}
        path = os.path.join(self.endpoint, build_url(fnc, *args), '')
        if 'params' in kwargs:
            path = "%s?%s" % (path, urllib.urlencode(kwargs.pop('params')))

        if kwargs:
            body = json.dumps(kwargs)
            method = 'POST'
        else:
            body = None
            method = 'GET'

        response, content = self.connection.request(path, method=method, body=body, headers=headers)
        if response.status == 201:
            return {'status': 201,
                    'success': 'created'}
        if response.status == 401:
            raise ClientUnauthorizedExecption()
        elif response.status == 403:
            raise ClientForbiddenException()
        elif response.status == 404:
            raise ClientNotFoundException()
        elif response.status == 500:
            raise ClientServerErrorExecption()

        try:
            return json.loads(content)
        except Exception:
            return {'status': response.status,
                    'error': 'unparseable response',
                    'data': content}

    def close(self):
        """
        Close API Client.

        Delete the connection(pool)
        """
        del(self.connection)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# encoding: utf-8
"""
protocols.py - handle data confoming the the Hudora standard protocols.
See http://github.com/hudora/huTools/tree/master/doc/standards for details.

Created by Maximillian Dornseif on 2009-12-27.
Copyright (c) 2009, 2011 HUDORA. All rights reserved.

Code is based on http://code.activestate.com/recipes/573463/
"""

import warnings
warnings.warn("huTools.protocols is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

from huTools.structured import make_struct

if __name__ == '__main__':
    print "NICHT MEHR NUTZEN!"
    make_struct()



#!/usr/bin/env python
# encoding: utf-8
"""
huTools/http/__init__.py

Provides a simple interface for doing HTTP requests on stock Python and on Google Appengine. Provides
unicode aware application/x-www-form-urlencoded encoding and an do multipart/form-data (file upload)
without aditional code. On Google Appengine it incerass the timeout from 5 seconds to 10.

Usage is without suprises::

    >>> status, header, body = fetch('http://www.postbin.org/o0ds54',
                                     {'küh': 'Iñtërnâtiônàlizætiøn', 'just a test': 212},
                                     headers={'X-Foo': 'Bar'})

File Upload just works::

    >>> status, header, body = fetch('http://www.postbin.org/o0ds54',
                                     {'hosts': open('/etc/hosts', 'r')}, 'POST')

"""

# Created by Maximillian Dornseif on 2010-10-24.
# Copyright (c) 2010 HUDORA. All rights reserved.

from huTools.http import tools
import cgi
import poster_encode
import urlparse

try:
    import engine_httplib2
    request = engine_httplib2.request
except ImportError:
    import engine_appengine
    request = engine_appengine.request


def fetch(url, content='', method='GET', credentials=None, headers=None, multipart=False, ua='', timeout=25):
    """Does a HTTP request with method `method` to `url`.

    Returns (status, headers, content) whereas `status` is an integer status code, `headers` is a dict
    containing the headers sent by the server and `content` is the body of the http response.

    Parameters to fetch are::

    * `url` is the fully qualified request URL. It may contain query prameters.
    * `content` is the request body to be sent. It may be a dict which for all requests expect POST
      is converted to query parameters. If there are query parameters already in the `url` they are merged
      with `content`. For POST requests the data is encoded as application/x-www-form-urlencoded
      or multipart/form-data and encoded. If the parameter `multipart` is `True` or if one of the values
      in content has a `name` attribute (which is the case for file objects) multipart encoding is choosen.
    * `headers` is a dict of header values
    * `credentials` can be a user:password combination
    * `ua` should be an additional User Agent string
    * `timeout` is the maximum number of seconds the request might take. This is advisory and may not be
       enforced.
    """

    myheaders = {'Accept-Encoding': 'gzip;q=1.0, *;q=0',
                 'User-Agent': '%s/huTools.http (gzip)' % ua}
    if headers:
        myheaders.update(headers)
    if method == 'POST':
        if hasattr(content, 'items'):
            # we assume content is a dict which needs to be encoded
            # decide to use multipart/form-data encoding or application/x-www-form-urlencoded
            for v in content.values():
                if hasattr(v, 'read'):  # file() or StringIO()
                    multipart = True
            if multipart:
                datagen, mp_headers = poster_encode.multipart_encode(content)
                myheaders.update(mp_headers)
                content = "".join(datagen)
            else:
                myheaders.update({'Content-Type': 'application/x-www-form-urlencoded'})
                content = tools.urlencode(content)
    else:
        # url parmater encoding
        if hasattr(content, 'items'):
            scheme, netloc, path, params, query, fragment = urlparse.urlparse(url)
            qdict = cgi.parse_qs(query)
            # ugly Unicode issues, see http://bugs.python.org/issue1712522
            qdict.update(content)
            query = tools.urlencode(qdict)
            url = urlparse.urlunparse((scheme, netloc, path, params, query, fragment))
            content = ''
    # convert all header values to strings (what about unicode?)
    for k, v in myheaders.items():
        myheaders[k] = str(v)
    # add authentication
    if credentials and not 'Authorization' in myheaders.keys():
        authheader = "Basic %s" % credentials.encode('base64').strip()
        myheaders["Authorization"] = authheader

    return request(url, method, content, myheaders, timeout)



# Copyright (C) 2008-2009 John Millikin <jmillikin@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""JSON parser/serializer for Python.

The main implementation of this module is accessed through calls to
``read()`` and ``write()``. See their documentation for details.

"""
__author__ = "John Millikin <jmillikin@gmail.com>"
__version__ = (1, 6, 1)
__license__ = "GPL"

__all__ = [
        'loads', 'dumps', 'read', 'write',
        'ReadError', 'WriteError', 'UnknownSerializerError',
]

import codecs
from decimal import Decimal
import re
import sys
from UserString import UserString

# Constants {{{
KEYWORDS = (('null', None), ('true', True), ('false', False))
try:
    INFINITY = float('inf')
except ValueError:
    INFINITY = 1e300000
try:
    NAN = float('nan')
except ValueError:
    NAN = INFINITY / INFINITY

UNICODE_BOMS = [
        (codecs.BOM_UTF32_BE, 'utf-32-be'),
        (codecs.BOM_UTF32_LE, 'utf-32-le'),
        (codecs.BOM_UTF16_BE, 'utf-16-be'),
        (codecs.BOM_UTF16_LE, 'utf-16-le'),
        (codecs.BOM_UTF8, 'utf-8'),
]
UTF_HEADERS = [
        ((0, 0, 0, 1), 'utf-32-be'),
        ((1, 0, 0, 0), 'utf-32-le'),
        ((0, 1, 0, 1), 'utf-16-be'),
        ((1, 0, 1, 0), 'utf-16-le'),
]

NUMBER_SPLITTER = re.compile(
        '^(?P<minus>-)?(?P<int>0|[1-9][0-9]*)'  # Basic integer portion
        '(?:\\.(?P<frac>[0-9]+))?'              # Fractional portion
        '(?P<exp>[eE][-+]?[0-9]+)?$',           # Exponent
)

READ_ESCAPES = {
        '\\': '\\',
        '"': '"',
        '/': '/',
        'b': '\b',
        'f': '\f',
        'n': '\n',
        'r': '\r',
        't': '\t',
}

WRITE_ESCAPES = {
        # Escaping the solidus is a security measure intended for
        # protecting users from broken browser parsing, if the consumer
        # is stupid enough to parse JSON by including it directly into
        # a <script> tag.
        #
        # See: http://t3.dotgnu.info/blog/insecurity/quotes-dont-help.html
        #'/': '\\/', - but WE don't do that (md)
        '"': '\\"',
        '\t': '\\t',
        '\b': '\\b',
        '\n': '\\n',
        '\r': '\\r',
        '\f': '\\f',
        '\\': '\\\\'
}


for __char_ord in range(0, 0x20):
    WRITE_ESCAPES.setdefault(unichr(__char_ord), '\\u%04x' % __char_ord)

ALLOWED_WHITESPACE = u'\u0020\u0009\u000A\u000D'
# }}}


# Exception classes {{{
class ReadError(ValueError):
    """Exception raised if there is an error parsing a JSON expression."""
    pass


class WriteError(ValueError):
    """Exception raised if there is an error generating a JSON expression."""
    pass


class UnknownSerializerError(WriteError):
    """Exception raised if there is no known way to convert a
    value to a JSON expression.

    """
    pass
# }}}


# Parser {{{
def unicode_autodetect_encoding(bytestring):
    """Intelligently convert a byte string to Unicode.

    Assumes the encoding used is one of the UTF-* variants. If the
    input is already in unicode, this is a noop.

    """
    if isinstance(bytestring, unicode):
        return bytestring

    # Check for UTF byte order marks in the bytestring
    for bom, encoding in UNICODE_BOMS:
        if bytestring.startswith(bom):
            return bytestring[len(bom):].decode(encoding)

    # Autodetect UTF-* encodings using the algorithm in the RFC
    # Don't use inline if..else for Python 2.4
    header = tuple((ord(b) and 1) or 0 for b in bytestring[:4])
    for utf_header, encoding in UTF_HEADERS:
        if header == utf_header:
            return bytestring.decode(encoding)

    # Default to UTF-8
    return bytestring.decode('utf-8')


class ParseErrorHelper(object):
    """Small class to provide a collection of error-formatting routines
    shared between the Python and C implementation.

    """
    def next_char_ord(self, text, offset):
        value = ord(text[offset])
        if (0xD800 <= value <= 0xDBFF) and len(text) >= 2:
            upper = value
            lower = ord(text[offset + 1])
            upper -= 0xD800
            lower -= 0xDC00
            value = ((upper << 10) + lower) + 0x10000

        if value > 0xffff:
            return "U+%08X" % value
        return "U+%04X" % value

    def generic(self, text, offset, message):
        line = text.count('\n', 0, offset) + 1
        if line == 1:
            column = offset + 1
        else:
            column = offset - text.rindex('\n', 0, offset)

        template = ("JSON parsing error at line %d, column %d"
                    " (position %d): %s")
        error = template % (line, column, offset, message)
        raise ReadError(error)

    def unexpected(self, text, offset, looking_for):
        char_ord = self.next_char_ord(text, offset)
        if looking_for is None:
            desc = "Unexpected %s." % (char_ord,)
        else:
            desc = "Unexpected %s while looking for %s." % (char_ord, looking_for)
        self.generic(text, offset, desc)

    def extra_data(self, text, offset):
        self.generic(text, offset, "Extra data after JSON expression.")

    def no_expression(self, text, offset):
        self.generic(text, offset, "No expression found.")

    def unknown_escape(self, text, offset, escape):
        self.generic(text, offset, "Unknown escape code: \\%s." % escape)

    def unterminated_unicode(self, text, offset):
        self.generic(text, offset, "Unterminated unicode escape.")

    def unterminated_string(self, text, offset):
        self.generic(text, offset, "Unterminated string.")

    def reserved_code_point(self, text, offset, ord):
        self.generic(text, offset, "U+%04X is a reserved code point." % ord)

    def missing_surrogate(self, text, offset):
        self.generic(text, offset, "Missing surrogate pair half.")

    def invalid_number(self, text, offset):
        self.generic(text, offset, "Invalid number.")


class Parser(object):
    def __init__(self, text, use_float, error_helper):
        self.text = text
        self.index = 0
        self.use_float = use_float
        self.raise_ = error_helper

    def parse(self):
        value = self.parse_raw(True)
        self.skip_whitespace()
        if self.index != len(self.text):
            self.raise_.extra_data(self.text, self.index)
        return value

    def parse_raw(self, root=False):
        self.skip_whitespace()
        if self.index == len(self.text):
            self.raise_.no_expression(self.text, 0)
        c = self.text[self.index]
        if c == '{':
            return self.read_object()
        if c == '[':
            return self.read_array()
        if root:
            self.raise_unexpected()
        if c == '"':
            return self.read_string()
        if c in 'tfn':
            return self.read_keyword()
        if c in '-0123456789':
            return self.read_number()
        self.raise_unexpected()

    def read_object(self):
        retval = {}
        start = self.index
        skip = lambda: self.skip_whitespace(start, "Unterminated object.")
        c = lambda: self.text[self.index]

        self.skip('{', "object start")
        skip()
        if c() == '}':
            self.skip('}', "object end")
            return retval
        while True:
            skip()
            if c() != '"':
                self.raise_unexpected("property name")
            key = self.parse_raw()
            skip()
            self.skip(':', "colon")
            skip()
            value = self.parse_raw()
            retval[key] = value
            skip()
            if c() == '}':
                self.skip('}', "object end")
                return retval
            self.skip(',', "comma")

    def read_array(self):
        retval = []
        start = self.index
        skip = lambda: self.skip_whitespace(start, "Unterminated array.")
        c = lambda: self.text[self.index]

        self.skip('[', "array start")
        skip()
        if c() == ']':
            self.skip(']', "array end")
            return retval
        while True:
            skip()
            value = self.parse_raw()
            retval.append(value)
            skip()
            if c() == ']':
                self.skip(']', "array end")
                return retval
            self.skip(',', "comma")

    def read_string(self):
        text_len = len(self.text)
        start = self.index
        escaped = False
        chunks = []

        self.skip('"', "string start")
        while True:
            while not escaped:
                if self.index >= text_len:
                    self.raise_.unterminated_string(self.text, start)

                c = self.text[self.index]
                if c == '\\':
                    escaped = True
                elif c == '"':
                    self.skip('"', "string end")
                    return u''.join(chunks)
                elif ord(c) < 0x20:
                    self.raise_unexpected()
                else:
                    chunks.append(c)
                self.index += 1

            escaped = False
            if self.index >= text_len:
                self.raise_.unterminated_string(self.text, start)

            c = self.text[self.index]
            if c == 'u':
                unescaped = self.read_unicode_escape()
                chunks.append(unescaped)
            elif c in READ_ESCAPES:
                chunks.append(READ_ESCAPES[c])
            else:
                self.raise_.unknown_escape(self.text, self.index - 1, c)
            self.index += 1

    def read_unicode_escape(self):
        """Read a JSON-style Unicode escape.

        Unicode escapes may take one of two forms:

        * \\uUUUU, where UUUU is a series of four hexadecimal digits that
        indicate a code point in the Basic Multi-lingual Plane.

        * \\uUUUU\\uUUUU, where the two points encode a UTF-16 surrogate pair.
          In builds of Python without wide character support, these are
          returned as a surrogate pair.

        """
        first_hex_str = self.text[self.index + 1:self.index + 5]
        if len(first_hex_str) < 4 or '"' in first_hex_str:
            self.raise_.unterminated_unicode(self.text, self.index - 1)
        first_hex = int(first_hex_str, 16)

        # Some code points are reserved for indicating surrogate pairs
        if 0xDC00 <= first_hex <= 0xDFFF:
            self.raise_.reserved_code_point(self.text, self.index - 1, first_hex)

        # Check if it's a UTF-16 surrogate pair
        if not(0xD800 <= first_hex <= 0xDBFF):
            self.index += 4
            return unichr(first_hex)

        second_hex_str = self.text[self.index + 5:self.index + 11]
        if(not(len(second_hex_str) >= 6
                and second_hex_str.startswith('\\u'))
            or '"' in second_hex_str):
            self.raise_.missing_surrogate(self.text, self.index + 5)

        second_hex = int(second_hex_str[2:], 16)
        if sys.maxunicode <= 65535:
            retval = unichr(first_hex) + unichr(second_hex)
        else:
            # Convert to 10-bit halves of the 20-bit character
            first_hex -= 0xD800
            second_hex -= 0xDC00

            # Merge into 20-bit character
            retval = unichr((first_hex << 10) + second_hex + 0x10000)
        self.index += 10
        return retval

    def read_keyword(self):
        for text, value in KEYWORDS:
            end = self.index + len(text)
            if self.text[self.index:end] == text:
                self.index = end
                return value
        self.raise_unexpected()

    def read_number(self):
        allowed = '0123456789-+.eE'
        end = self.index
        try:
            while self.text[end] in allowed:
                end += 1
        except IndexError:
            pass
        match = NUMBER_SPLITTER.match(self.text[self.index:end])
        if not match:
            self.raise_.invalid_number(self.text, self.index)

        self.index = end
        int_part = int(match.group('int'), 10)
        if match.group('frac') or match.group('exp'):
            if self.use_float:
                return float(match.group(0))
            return Decimal(match.group(0))
        if match.group('minus'):
            return -int_part
        return int_part

    def skip(self, text, error):
        new_index = self.index + len(text)
        skipped = self.text[self.index:new_index]
        if skipped != text:
            self.raise_unexpected(error)
        self.index = new_index

    def skip_whitespace(self, start=None, err=None):
        text_len = len(self.text)
        ws = '\x09\x20\x0a\x0d'
        while self.index < text_len and self.text[self.index] in ws:
            self.index += 1
        if self.index >= text_len and (start is not None) and (err is not None):
            self.raise_.generic(self.text, start, err)

    def raise_unexpected(self, message=None):
        self.raise_.unexpected(self.text, self.index, message)


def read_impl(text, use_float, error_helper, decimal):
    parser = Parser(text, use_float, error_helper)
    return parser.parse()


def read(bytestring, use_float=False):
    """Parse a JSON expression into a Python value.

    If string is a byte string, it will be converted to Unicode
    before parsing (see unicode_autodetect_encoding).

    """
    text = unicode_autodetect_encoding(bytestring)
    return read_impl(text, use_float, ParseErrorHelper(), Decimal)

loads = read
# }}}


# Serializer {{{
ATOMIC_TYPES = (
          type(None), bool, int, long, float, complex, Decimal, unicode, str)


def is_iterable(x):
    try:
        iter(x)
        return True
    except TypeError:
        return False


class SerializerErrorHelper(object):
    def invalid_root(self):
        raise WriteError("The outermost container must be an array or object.")

    def unknown_serializer(self, value):
        raise UnknownSerializerError("No known serializer for object: %r" % (value,))

    def self_referential(self):
        raise WriteError("Cannot serialize self-referential values.")

    def invalid_object_key(self):
        raise WriteError("Only strings may be used as object keys.")

    def incomplete_surrogate(self):
        raise WriteError("Cannot serialize incomplete surrogate pair.")

    def invalid_surrogate(self):
        raise WriteError("Cannot serialize invalid surrogate pair.")

    def reserved_code_point(self, ord):
        raise WriteError("Cannot serialize reserved code point U+%04X." % ord)

    def no_nan(self):
        raise WriteError("Cannot serialize NaN.")

    def no_infinity(self):
        raise WriteError("Cannot serialize Infinity.")

    def no_neg_infinity(self):
        raise WriteError("Cannot serialize -Infinity.")

    def no_imaginary(self):
        raise WriteError("Cannot serialize complex numbers with"
                         " imaginary components.")


class Serializer(object):
    def __init__(self, sort_keys, indent, ascii_only,
                  coerce_keys, encoding, on_unknown,
                  error_helper):
        self.sort_keys = sort_keys
        self.indent = indent
        self.ascii_only = ascii_only
        self.coerce_keys = coerce_keys
        self.encoding = encoding
        self.on_unknown = on_unknown
        self.raise_ = error_helper

    def append(self, value):
        raise NotImplementedError

    def serialize(self, value):
        raise NotImplementedError

    def serialize_object(self, value, parent_ids, in_unknown_hook=False):
        if isinstance(value, UserString):
            value = value.data
        if isinstance(value, ATOMIC_TYPES):
            if not parent_ids:
                self.raise_.invalid_root()
            self.serialize_atom(value)
        elif hasattr(value, 'items'):
            self.serialize_mapping(value, parent_ids)
        elif is_iterable(value):
            self.serialize_iterable(value, parent_ids)
        elif not in_unknown_hook:
            new_value = self.on_unknown(value,
                    self.raise_.unknown_serializer)
            self.serialize_object(new_value, parent_ids, True)
        else:
            self.raise_.unknown_serializer(value)

    def get_separators(self, indent_level):
        if self.indent is None:
            return '', ''
        else:
            indent = '\n' + (self.indent * (indent_level + 1))
            post_indent = '\n' + (self.indent * indent_level)
            return indent, post_indent

    def serialize_mapping(self, value, parent_ids):
        v_id = id(value)
        if v_id in parent_ids:
            self.raise_.self_referential()

        a = self.append
        first = True
        items = value.items()
        if self.sort_keys:
            items = sorted(items)

        indent, post_indent = self.get_separators(len(parent_ids))

        a('{')
        for key, item in items:
            if isinstance(key, UserString):
                key = key.data
            if not isinstance(key, (str, unicode)):
                if self.coerce_keys:
                    key = unicode(key)
                else:
                    self.raise_.invalid_object_key()
            if first:
                first = False
            else:
                a(',')
            a(indent)
            self.serialize_object(key, parent_ids + [v_id])
            if self.indent is None:
                a(':')
            else:
                a(': ')
            self.serialize_object(item, parent_ids + [v_id])
        a(post_indent)
        a('}')

    def serialize_iterable(self, value, parent_ids):
        v_id = id(value)
        if v_id in parent_ids:
            self.raise_.self_referential()

        a = self.append

        indent, post_indent = self.get_separators(len(parent_ids))

        a('[')
        first = True
        for item in value:
            if first:
                first = False
            else:
                a(',')
            a(indent)
            self.serialize_object(item, parent_ids + [v_id])
        a(post_indent)
        a(']')

    def serialize_atom(self, value):
        for keyword, kw_value in KEYWORDS:
            if value is kw_value:
                return self.append(keyword)

        if isinstance(value, unicode):
            self.serialize_unicode(value)
        elif isinstance(value, str):
            self.serialize_bytes(value)
        elif isinstance(value, (int, long)):
            self.append(unicode(value))
        elif isinstance(value, float):
            self.serialize_float(value)
        elif isinstance(value, complex):
            self.serialize_complex(value)
        elif isinstance(value, Decimal):
            self.serialize_decimal(value)
        else:
            self.raise_.unknown_serializer(value)

    def serialize_bytes(self, value):
        self.serialize_unicode(unicode(value, 'ascii'))

    def serialize_unicode(self, value):
        a = self.append
        stream = iter(value)
        a('"')
        for char in stream:
            ochar = ord(char)
            if char in WRITE_ESCAPES:
                a(WRITE_ESCAPES[char])
            elif ochar > 0x7E:
                # Prevent invalid surrogate pairs from being
                # encoded.
                if 0xD800 <= ochar <= 0xDBFF:
                    try:
                        nextc = stream.next()
                    except StopIteration:
                        self.raise_.incomplete_surrogate()
                    onext = ord(nextc)
                    if not(0xDC00 <= onext <= 0xDFFF):
                        self.raise_.invalid_surrogate()
                    if self.ascii_only:
                        a('\\u%04x\\u%04x' % (ochar, onext))
                    else:
                        a(char)
                        a(nextc)
                elif 0xDC00 <= ochar <= 0xDFFF:
                    self.raise_.reserved_code_point(ochar)
                elif self.ascii_only:
                    if ochar > 0xFFFF:
                        unicode_value = ord(char)
                        reduced = unicode_value - 0x10000
                        second_half = (reduced & 0x3FF)  # Lower 10 bits
                        first_half = (reduced >> 10)

                        first_half += 0xD800
                        second_half += 0xDC00

                        a('\\u%04x\\u%04x' % (first_half, second_half))
                    else:
                        a('\\u%04x' % ochar)
                else:
                    a(char)
            else:
                a(char)

        a('"')

    def serialize_float(self, value):
        if value != value:
            self.raise_.no_nan()
        if value == INFINITY:
            self.raise_.no_infinity()
        if value == -INFINITY:
            self.raise_.no_neg_infinity()
        self.append(repr(value))

    def serialize_complex(self, value):
        if value.imag == 0.0:
            self.append(repr(value.real))
        else:
            self.raise_.no_imaginary()

    def serialize_decimal(self, value):
        if value != value:
            self.raise_.no_nan()
        s_value = unicode(value)
        if s_value == u'Infinity':
            self.raise_.no_infinity()
        elif s_value == u'-Infinity':
            self.raise_.no_neg_infinity()
        self.append(s_value)


class StreamSerializer(Serializer):
    def __init__(self, fp, *args, **kwargs):
        super(StreamSerializer, self).__init__(*args, **kwargs)
        self.fp = fp

    def append(self, value):
        if isinstance(value, str):
            value = unicode(value, 'ascii')
        if self.encoding is not None:
            value = value.encode(self.encoding)
        self.fp.write(value)

    def serialize(self, value):
        self.serialize_object(value, [])


class BufferSerializer(Serializer):
    def __init__(self, *args, **kwargs):
        super(BufferSerializer, self).__init__(*args, **kwargs)
        self.chunks = []

    def append(self, value):
        self.chunks.append(value)

    def serialize(self, value):
        self.serialize_object(value, [])
        str_result = u''.join(self.chunks)
        if self.encoding is None:
            return str_result
        return str_result.encode(self.encoding)


def dump_impl(value, fp, sort_keys, indent, ascii_only,
               coerce_keys, encoding, on_unknown, error_helper,
               decimal, userstring):
    serializer = StreamSerializer(fp, sort_keys, indent, ascii_only,
                                   coerce_keys, encoding,
                                   on_unknown, error_helper)
    return serializer.serialize(value)


def dump(value, fp, sort_keys=False, indent=None, ascii_only=True,
          coerce_keys=False, encoding='utf-8', on_unknown=None):
    """Serialize a Python value to a JSON-formatted byte string.

    Rather than being returned as a string, the output is written to
    a file-like object.

    """
    return dump_impl(value, fp, sort_keys,
                      validate_indent(indent), ascii_only,
                      coerce_keys, encoding,
                      validate_on_unknown(on_unknown),
                      SerializerErrorHelper(),
                      UserString, Decimal)


def write_impl(value, sort_keys, indent, ascii_only,
                coerce_keys, encoding, on_unknown, error_helper,
                decimal, userstring):
    serializer = BufferSerializer(sort_keys, indent, ascii_only,
                                   coerce_keys, encoding,
                                   on_unknown, error_helper)
    return serializer.serialize(value)


def write(value, sort_keys=False, indent=None, ascii_only=True,
           coerce_keys=False, encoding='utf-8', on_unknown=None):
    """Serialize a Python value to a JSON-formatted byte string.

    .. describe:: value

            The Python object to serialize.

    .. describe:: sort_keys

            Whether object keys should be kept sorted. Useful
            for tests, or other cases that check against a
            constant string value.

    .. describe:: indent

            A string to be used for indenting arrays and objects.
            If this is non-None, pretty-printing mode is activated.

    .. describe:: ascii_only

            Whether the output should consist of only ASCII
            characters. If this is True, any non-ASCII code points
            are escaped even if their inclusion would be legal.

    .. describe:: coerce_keys

            Whether to coerce invalid object keys to strings. If
            this is False, an exception will be raised when an
            invalid key is specified.

    .. describe:: encoding

            The output encoding to use. This must be the name of an
            encoding supported by Python's codec mechanism. If
            None, a Unicode string will be returned rather than an
            encoded bytestring.

            If a non-UTF encoding is specified, the resulting
            bytestring might not be readable by many JSON libraries,
            including jsonlib.

            The default encoding is UTF-8.
    .. describe:: on_unknown

            A callable to be used for converting objects of an
            unrecognized type into a JSON expression. If ``None``,
            unrecognized objects will raise an ``UnknownSerializerError``.

    """
    return write_impl(value, sort_keys, validate_indent(indent), ascii_only,
                       coerce_keys, encoding,
                       validate_on_unknown(on_unknown),
                       SerializerErrorHelper(),
                       UserString, Decimal)

dumps = write


def validate_indent(indent):
    if indent is not None:
        indent = unicode(indent)
    if not(indent is None or len(indent) == 0):
        if len(indent.strip(ALLOWED_WHITESPACE)) > 0:
            raise TypeError("Only whitespace may be used for indentation.")
    return indent


def validate_on_unknown(f):
    def on_unknown(value, unknown):
        if f is not None:
            return f(value)
        unknown(value)
    if not(f is None or callable(f)):
        raise TypeError("The on_unknown object must be callable.")
    return on_unknown
# }}}

try:
    from _jsonlib import read_impl, write_impl, dump_impl
except ImportError:
    pass



#!/usr/bin/env python
# encoding: utf-8
"""
xml.py

Created by Christian Klein on 2010-02-26.
Copyright (c) 2010 HUDORA GmbH. All rights reserved.
"""

import couchdb.client


def setup_couchdb(servername, database):
    """Get a connection handler to the CouchDB Database, creating it when needed."""

    server = couchdb.client.Server(servername)
    if database in server:
        return server[database]
    else:
        return server.create(database)



#!/usr/bin/env python
# encoding: utf-8
"""
xmltools.py

Created by Christian Klein on 2010-02-26.
Copyright (c) 2010 HUDORA GmbH. All rights reserved.
"""

import datetime
import xml.etree.ElementTree as ET


def encode_text(data):
    """
    Encode for usage in XML Tree

    >>> encode_text(None)
    u''
    >>> encode_text('Alex')
    u'Alex'
    >>> encode_text(u'Alex')
    u'Alex'
    >>> encode_text(12)
    u'12'
    >>> encode_text(callable(encode_text))
    u'True'
    >>> encode_text(callable)
    u''
    """

    if callable(data):
        try:
            return encode_text(data())
        except TypeError:
            return u''

    if isinstance(data, str):
        return data.decode('utf-8', 'replace')
    elif isinstance(data, datetime.datetime):
        if not(data.hour or data.minute):
            fmt = '%Y-%m-%d'
        else:
            fmt = '%Y-%m-%d %H:%M'
        return data.strftime(fmt)
    elif data is None:
        return u''
    else:
        return unicode(data)


def add_fields(root, source, fieldnames):
    """
    Add a number of fields to a XML Tree.
    """
    for fieldname in fieldnames:
        elem = ET.SubElement(root, fieldname)
        if hasattr(source, fieldname):
            elem.text = encode_text(getattr(source, fieldname, ''))
    return root


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# encoding: utf-8
"""
formats.py - formating and parsing of timestamps.

Created by Maximillian Dornseif on 2007-06-24.
Copyright (c) 2007, 2010 HUDORA GmbH. All rights reserved.
"""

import unittest
import datetime
import time
import email.utils


def rfc3339_date(date=None):
    """Formates a datetime object according to RfC 3339."""
    date = date or datetime.datetime.now()
    return date.strftime('%Y-%m-%dT%H:%M:%SZ')


def rfc3339_date_parse(date):
    """Parses an RfC 3339 timestamp into a datetime object."""
    return datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%SZ')


def convert_to_date(date):
    """
    Converts argument into a date object.

    Assumes argument to be a RfC 3339 coded date or a date(time) object.
    """

    if hasattr(date, 'date') and callable(date.date):
        # e.g. datetime objects
        return date.date()
    elif isinstance(date, datetime.date):
        return date
    elif isinstance(date, basestring):
        date = date[:10]  # strip timestamp
        try:
            return datetime.datetime.strptime(date, '%Y-%m-%d').date()
        except ValueError:
            return datetime.datetime.strptime(date, '%Y%m%d').date()
    elif not date:
        return None
    raise ValueError("Unknown value %r (%s)" % (date, type(date)))


def convert_to_datetime(date):
    """Converts argument into a datetime object.

    Assumes argument to be a RfC 3339 coded date or a date(time) object.
    """

    if isinstance(date, datetime.datetime):
        return date
    elif isinstance(date, datetime.date):  # order mattes! datetime is a subclass of date
        return datetime.datetime(date.year, date.month, date.day)
    elif isinstance(date, basestring):
        if len(date) < 11:
            return convert_to_datetime(convert_to_date(date))
        else:
            ms = 0
            if '.' in date:
                date, ms = date.split('.')
            date = date.rstrip('Z')
            try:
                ret = datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%S')
            except ValueError:
                try:
                    ret = datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
                except ValueError:
                    ret = datetime.datetime.strptime(date, '%Y%m%dT%H%M%S')
            if ms:
                return datetime.datetime(ret.year, ret.month, ret.day,
                                         ret.hour, ret.minute, ret.second, int(ms))
            return ret
    elif not date:
        return None
    raise ValueError("Unknown value %r (%s)" % (date, type(date)))


def rfc2616_date(date=None):
    """Formates a datetime object according to RfC 2616.

    RfC 2616 is a subset of RFC 1123 date.
    Weekday and month names for HTTP date/time formatting; always English!
    """
    date = date or datetime.datetime.now()
    return email.utils.formatdate(time.mktime(date.timetuple()), usegmt=True)


def rfc2616_date_parse(data):
    """Parses an RfC 2616/2822 timestapm into a datetime object."""
    return datetime.datetime.fromtimestamp(email.utils.mktime_tz(email.utils.parsedate_tz(data)))


class _FormatsTests(unittest.TestCase):

    def test_rfc3339_date(self):
        """Test basic rfc3339_date output."""
        self.assertEqual(rfc3339_date(datetime.datetime(2007, 2, 3, 4, 5, 6)), '2007-02-03T04:05:06Z')

    def test_rfc3339_date_parse(self):
        """Test basic rfc3339_date_parse output."""
        self.assertEqual(rfc3339_date_parse('2007-02-03T04:05:06Z'),
                         datetime.datetime(2007, 2, 3, 4, 5, 6))

    def test_rfc2616_date(self):
        """Test basic rfc2616_date output."""
        self.assertEqual(rfc2616_date(datetime.datetime(2007, 2, 3, 4, 5, 6)),
                         'Sat, 03 Feb 2007 03:05:06 GMT')

    def test_rfc2616_date_parse(self):
        """Test basic rfc2616_date_parse output."""
        self.assertEqual(rfc2616_date_parse('Sat, 03 Feb 2007 03:05:06 GMT'),
                         datetime.datetime(2007, 2, 3, 4, 5, 6))

    def test_convert_to_datetime(self):
        """Test convert_to_datetime() and convert_to_date() functionality"""
        self.assertEqual(convert_to_datetime(datetime.date(2007, 2, 3)),
                                             datetime.datetime(2007, 2, 3, 0, 0))
        self.assertEqual(convert_to_datetime(datetime.datetime(2007, 2, 3, 13, 14, 15, 16)),
                                             datetime.datetime(2007, 2, 3, 13, 14, 15, 16))
        self.assertEqual(convert_to_datetime('2007-02-03'), datetime.datetime(2007, 2, 3, 0, 0))
        self.assertEqual(convert_to_datetime('2007-2-3'), datetime.datetime(2007, 2, 3, 0, 0))
        self.assertEqual(convert_to_datetime('20070203'), datetime.datetime(2007, 2, 3, 0, 0))
        self.assertEqual(convert_to_datetime('20070203T131415'), datetime.datetime(2007, 2, 3, 13, 14, 15))
        self.assertEqual(convert_to_datetime('2007-02-03T13:14:15'),
                         datetime.datetime(2007, 2, 3, 13, 14, 15))
        self.assertEqual(convert_to_datetime('2007-02-03T13:14:15.16'),
                         datetime.datetime(2007, 2, 3, 13, 14, 15, 16))
        self.assertEqual(convert_to_datetime('2007-02-03 13:14:15'),
                         datetime.datetime(2007, 2, 3, 13, 14, 15))
        self.assertEqual(convert_to_datetime('2007-02-03 13:14:15.16'),
                         datetime.datetime(2007, 2, 3, 13, 14, 15, 16))


class _ApiTests(unittest.TestCase):

    def test_defaults(self):
        """Test rfc3339_date defaults"""
        rfc3339_date()
        rfc2616_date()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
decorators.py

Created by Maximillian Dornseif on 2007-05-10.
Copyright (c) 2007 HUDORA GmbH. All rights reserved.
"""


import cPickle as pickle
import functools
import hashlib
from _decorator import decorator
# TODO: können wir _decorator durch "from functools import wraps" ersetzen?


def _getattr_(obj, name, default_thunk):
    "Similar to .setdefault in dictionaries."
    try:
        return getattr(obj, name)
    except AttributeError:
        default = default_thunk()
        setattr(obj, name, default)
        return default

# from http://www.phyast.pitt.edu/~micheles/python/documentation.html


@decorator
def memoize(func, *args):
    """This decorator caches the results of the function it decorates.

    See http://en.wikipedia.org/wiki/Memoisation for fare more than ypou ever want to know on that topic.
    """

    dic = _getattr_(func, "memoize_dic", dict)
    # memoize_dic is created at the first call
    if args in dic:
        return dic[args]
    else:
        result = func(*args)
        dic[args] = result
        return result


# from somewhere on the internet

def cache_function(length):
    """
    A variant of the snippet posted by Jeff Wheeler at
    http://www.djangosnippets.org/snippets/109/

    Caches a function, using the function and its arguments as the key, and the return
    value as the value saved. It passes all arguments on to the function, as
    it should.

    The decorator itself takes a length argument, which is the number of
    seconds the cache will keep the result around.
    """

    def decorator(func):

        def inner_func(*args, **kwargs):
            from django.core.cache import cache

            raw = [func.__name__, func.__module__, args, kwargs]
            pickled = pickle.dumps(raw, protocol=pickle.HIGHEST_PROTOCOL)
            key = hashlib.md5(pickled).hexdigest()
            value = cache.get(key)
            if value:
                return value
            else:
                result = func(*args, **kwargs)
                cache.set(key, result, length)
                return result
        return inner_func
    return decorator


# from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/425445

def func_once(func):
    "A decorator that runs a function only once."

    def decorated(*args, **kwargs):
        try:
            return decorated._once_result
        except AttributeError:
            decorated._once_result = func(*args, **kwargs)
            return decorated._once_result
    return decorated


def method_once(method):
    "A decorator that runs a method only once."
    attrname = "_%s_once_result" % id(method)

    def decorated(self, *args, **kwargs):
        try:
            return getattr(self, attrname)
        except AttributeError:
            setattr(self, attrname, method(self, *args, **kwargs))
            return getattr(self, attrname)
    return decorated


def none_on_exception(func):
    """A decorator that returns the return value of a function or None if an exception is raised."""

    @functools.wraps(func)
    def _decorator(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception:
            return None
    return _decorator



#!/usr/bin/env python
# encoding: utf-8
"""
hutools/world - countries of the world (to be procise: countries we do buiseness with)

Most interesting are the constants COUNTRY_CHOICES and COUNTRIES.

COUNTRY_CHOICES = [('DE', 'Deutschland'), ('AT', u'Österreich'), ...]
COUNTRIES = ['DE', 'AT', ...]

Created by Maximillian Dornseif on 2007-05-01.
Copyright (c) 2007, 2010 HUDORA GmbH. BSD Licensed.
"""

import doctest
import sys

COUNTRY_CHOICES = [('DE', 'Deutschland')] \
                   + sorted([
                   ('AT', u'Österreich'),
                   ('CH', 'Schweiz'),
                   ('BE', 'Belgien'),
                   ('FR', 'Frankreich'),
                   ('ES', 'Spanien'),
                   ('NL', 'Niederlande'),
                   ('IT', 'Italien'),
                   ('GB', 'Grossbritannien'),
                   ('LI', 'Liechtenstein'),
                   ('LU', 'Luxemburg'),
                   ('PT', 'Portugal'),
                   ('CZ', 'Tschechien'),
                   ('SE', 'Schweden'),
                   ('GR', 'Griechenland'),
                   ('DK', 'Daenemark'),
                   ('US', 'USA'),
                   ('AR', 'Argentinien'),
                   ('HU', 'Ungarn'),
                   ('LV', 'Lettland'),
                   ('PL', 'Polen'),
                   ('LT', 'Litauen'),
                   ('SK', 'Slowakei'),
                   ('HR', 'Kroatien'),
                   ('IE', 'Irland'),
                   ('SI', 'Slowenien'),
                   ('EE', 'Estland'),
                   ('BG', 'Bulgarien'),
                   ('NO', 'Norwegen'),
                   ('AU', 'Australien'),
                   ('IL', 'Israel'),
                   ('FI', 'Finnland'),
                   ('RS', 'Republik Serbien'),
                   ('ZA', u'Südafrika'),
                   ('RO', u'Rumänien'),
                   ('MT', 'Malta'),
                   ('CY', 'Zypern'),
                  ])


COUNTRIES = dict(COUNTRY_CHOICES).keys()


# Stand: Mai 2009 - diese MÜSSEN auch in COUNTRY_CHOICES vorkommen.
EUROPEAN_UNION = ['BE', 'IT', 'RO', 'BG', 'LV', 'SE',
                  'DK', 'LT', 'SK', 'DE', 'LU', 'SI',
                  'EE', 'MT', 'ES', 'FI', 'NL', 'CZ',
                  'FR', 'AT', 'HU', 'GR', 'PL', 'GB',
                  'IE', 'PT', 'CY']


def in_european_union(isoland):
    """
    Gibt zurück, ob ein Land Mitglied der EU ist.

    >>> in_european_union('DE')
    True
    >>> in_european_union('CH')
    False
    >>> all(map(in_european_union, EUROPEAN_UNION))
    True
    >>> non_eu_countries = set((abrev for abrev, name in COUNTRY_CHOICES)) - set(EUROPEAN_UNION)
    >>> any(map(in_european_union, non_eu_countries))
    False
    """

    return isoland.upper() in EUROPEAN_UNION

if __name__ == "__main__":
    failure_count, test_count = doctest.testmod()
    sys.exit(failure_count)



#!/usr/bin/env python
# encoding: utf-8
"""
postmark.py - support for the postmark mail sending API

Based on pytohn-postmark (C) 2009-2010 David Martorana, Wildbit LLC, Python Software Foundation.

Created by Maximillian Dornseif on 2010-09-27.
Copyright (c) 2010 HUDORA. All rights reserved.
"""


import hujson
import logging
import urllib2


__POSTMARK_URL__ = 'http://api.postmarkapp.com/'


def send_mail(message, api_key=None):
        '''
        Send the email through the Postmark system.

        message is a dict containing:
        Sender:         Who the email is coming from, in either
                        "name@email.com" or "First Last <name@email.com>" format
        To:             Who to send the email to, in either
                        "name@email.com" or "First Last <name@email.com>" format
                        Can be multiple values separated by commas (limit 20)
        Subject:        Subject of the email

        optional elements:
        TextBody:       Email message in plain text
        HtmlBody:       Email message in HTML
        Cc:             Who to copy the email to, in either
                        "name@email.com" or "First Last <name@email.com>" format
                        Can be multiple values separated by commas (limit 20)
        Bcc:            Who to blind copy the email to, in either
                        "name@email.com" or "First Last <name@email.com>" format
                        Can be multiple values separated by commas (limit 20)
        Tag:            Use for adding categorizations to your email
        Attachments:    A list of attachments. Attachments can be either
                        a list of {name, data, content_type} dicts,
        '''

        if 'Sender' in message and ('From' not in message):
            message['From'] = message['Sender']
        for attr in 'From To Subject'.split():
            if attr not in message:
                raise RuntimeError('missing Parameter %s' % attr)
        if ('TextBody' not in message) and ('HtmlBody' not in message):
            raise RuntimeError('missing Body %r' % message)

        attachments = []
        for attachment in message.get('Attachments', []):
            attachments.append({
                    "Name": attachment['Name'],
                    "Content": attachment['Content'].encode('base64'),
                    "ContentType": attachment['ContentType'],
                    })
        if attachments:
            message['Attachments'] = attachments

        # Set up the url Request
        req = urllib2.Request(
            __POSTMARK_URL__ + 'email',
            hujson.dumps(message),
            {
                'Accept': 'application/json',
                'Content-Type': 'application/json',
                'X-Postmark-Server-Token': api_key,
            }
        )

        logging.debug('Accessing %semail' % __POSTMARK_URL__)
        try:
            result = urllib2.urlopen(req)
            result.close()
            if result.code == 200:
                return True
            else:
                raise RuntimeError('Postmark Return code %d: %s' % (result.code, result.msg))
        except urllib2.HTTPError, err:
            if err.code == 401:
                raise RuntimeError('Sending Unauthorized - incorrect API key.', err)
            elif err.code == 422:
                try:
                    jsontxt = err.read()
                    jsonobj = hujson.loads(jsontxt)
                    desc = jsonobj['Message']
                except:
                    desc = 'Description not given'
                raise RuntimeError('Unprocessable Entity: %s' % desc)
            elif err.code == 500:
                raise RuntimeError('Internal server error at Postmark. Admins have been alerted.', err)
        except urllib2.URLError, err:
            if hasattr(err, 'reason'):
                raise RuntimeError(("URLError: Failed to reach the server: %s (See 'inner_exception' for"
                                    " details)") % err.reason, err)
            elif hasattr(err, 'code'):
                raise RuntimeError(("URLError: %d: The server couldn't fufill the request. (See"
                                    " 'inner_exception' for details)") % err.code, err)
            else:
                raise RuntimeError("URLError: The server couldn't fufill the request. (See 'inner_exception'"
                                   " for details)", err)



# -*- coding: utf-8 -*-

"""Module for robust typecasts."""

from types import ListType, TupleType
import doctest
import re
import sys
import warnings


def only_digits(data):
    """Removes all non-digits from a string

    >>> only_digits('0123456789')
    '0123456789'
    >>> only_digits('ab123')
    '123'
    >>> only_digits('1abc')
    '1'
    >>> only_digits('abc')
    ''
    """

    return re.sub(r'[^0-9]', '', data)


def int_or_0(data, default=0):
    """Wandelt "None" in default, lässt alle anderen werte unverändert.

    >>> int_or_0(5)
    5
    >>> int_or_0(None)
    0
    >>> int_or_0(0)
    0
    >>> int_or_0(-1)
    -1
    >>> int_or_0([])
    0
    >>> int_or_0(tuple())
    0
    >>> int_or_0(None, default=0xaffe)
    45054
    """

    if not data:
        return default

    try:
        if type(data) in (ListType, TupleType):
            return int(data[0])
        return int(data)
    except TypeError:
        return default


def io0(data):
    warnings.warn("io0() is deprecated. Use int_or_0()", DeprecationWarning, stacklevel=2)
    return int_or_0(data)


def float_or_0(data, default=0.0):
    """Helper fnc. Returns data casted to a float value if possible, else to 0.

    >>> float_or_0(5.0)
    5.0
    >>> float_or_0(None)
    0.0
    >>> float_or_0(0)
    0.0
    >>> float_or_0(-1)
    -1.0
    >>> float_or_0([])
    0.0
    >>> float_or_0(tuple())
    0.0

    """
    if not data:
        return default
    try:
        return float(data)
    except (TypeError, ValueError):
        # FIXME do we really want to allow to convert anything to zero here or is it better, not to
        # catch the type and value errors an let casting fail?
        return default


if __name__ == '__main__':
    failure_count, test_count = doctest.testmod()
    sys.exit(failure_count)



#!/usr/bin/env python
# encoding: utf-8
"""
humessaging.py - tools for creating and halndling Messages complying with the Hudora Message Format.

The HUDORA Message format is a set of simple conventions to format messages in a
message system agnostic way. Messages are encoded in JSON.

Created by Maximillian Dornseif on 2009-07-29.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

import decimal
import doctest
import huTools.calendar.formats
import huTools.luids
import re
import simplejson as json
import warnings
import sys

if sys.version_info[:2] <= (2, 5):
    # ugly monkeypatch to make doctests work. For the reasons see
    # See http://mail.python.org/pipermail/python-dev/2008-July/081420.html
    # It can go away once all our boxes run python > 2.5
    decimal.Decimal.__repr__ = lambda s: "Decimal('%s')" % str(s)


def extend_audittrail(audit_info='', audit_trail=''):
    """Append audit_info to audit_trail.

    >>> extend_audittrail()
    ''
    >>> extend_audittrail('a;a')
    'a a'
    >>> extend_audittrail('b;b', 'a a')
    'a a;b b'
    """

    warnings.warn("hutools.humessaging is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

    return ';'.join([x for x in audit_trail.split(';') + [audit_info.replace(';', ' ')] if x])


def empty_message(creator, audit_info='', audit_trail='', guid=''):
    """Returns an empty message template following the huMessaging Standard.

    creator
      user (prefered) or library tool which created the message. Suitable for audit messages.
      Example: "Maximillian Dornseif via cs.zwitscher/r3456"
    audit_info
      a human readable description of the processing stage.
      Example: "flagged as problematic by nachschieber.py"
    audit_trail
      semicolen sparated list of previous processing stages.
    guid
      Unique id of this message. Should never repeat. Generate by something unique for your
      processing step. E.g. "lieferschenrueckmeldung-4234543". Should only contain
      printable ASCII characters.
    """

    warnings.warn("hutools.humessaging is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

    ret = {'created_at': huTools.calendar.formats.rfc3339_date(),
           'created_by': creator,
           'audit_trail': audit_trail,
           'guid': guid,
          }
    if audit_info:
        ret['audit_trail'] = extend_audittrail(audit_info, audit_trail)
    else:
        ret['audit_trail'] = extend_audittrail('send via %s' % creator, audit_trail)
    if not guid:
        ret['guid'] = 'msg-%s' % huTools.luids.guid128(salt=audit_info)
    return ret


_marker = '!2498313c6da44c02509d99642bada2d4!'  # to detect decimal


def _encode_decimal(obj):
    """Helper for encoding decimal objects. Needs later fixup by regular expressions.

    >>> _encode_decimal(decimal.Decimal('5.00'))
    '!2498313c6da44c02509d99642bada2d4!#5.00#!2498313c6da44c02509d99642bada2d4!'
    """
    if isinstance(obj, decimal.Decimal):
        return "%s#%s#%s" % (_marker, repr(obj).strip('Decimal()"\''), _marker)
    raise TypeError("%r is not JSON serializable" % (obj, ))


_decimal_re = \
    re.compile(r'"!2498313c6da44c02509d99642bada2d4!#(\d+)\.(\d+)#!2498313c6da44c02509d99642bada2d4!"')


def encode(message):
    """Decode the dict-like message into a valid wire representation.

    Besides the types familar from Python's json module it can serialize Decimal() objects.

    >>> encode({'guid': 123, 'created_by': 'test', 'num': decimal.Decimal('5.00')})
    '{"created_by":"test","guid":123,"num":5.00}'
    """

    warnings.warn("hutools.humessaging is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

    assert 'created_by' in message
    assert 'guid' in message
    ret = json.dumps(message, default=_encode_decimal, sort_keys=True, separators=(',', ':'))
    # replace "Decimal(\\"5.00\\")" with 5.00
    return re.sub(_decimal_re, r'\1.\2', ret)


def decode(data):
    """
    Decode a message from a valid wire representation.

    Besides the types familar from Python's json module it can unserialize Decimal() objects.

    >>> decode('{"created_by":"test","guid":123,"num":5.00}')
    {'guid': 123, 'num': Decimal('5.00'), 'created_by': 'test'}
    """

    warnings.warn("hutools.humessaging is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

    return json.loads(data, parse_float=decimal.Decimal)


def setup_queue(chan, name, durable=False, auto_delete=False, exclusive=False):
    """Sets up AMQP Queue 'name' and the assorted exchange, routing key and stuff dictated by AMQP"""

    warnings.warn("hutools.humessaging is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

    chan.exchange_declare(exchange=name, type="direct", durable=durable, auto_delete=auto_delete)
    chan.queue_declare(queue=name, durable=durable, exclusive=exclusive, auto_delete=auto_delete)
    chan.queue_bind(queue=name, exchange=name, routing_key=name)


if __name__ == "__main__":
    failure_count, test_count = doctest.testmod()
    sys.exit(failure_count)



#!/usr/bin/env python
# encoding: utf-8
"""
aggregation.py - implement things similar to "group by" in SQL.

Created by Maximillian Dornseif on 2010-03-07.
Copyright (c) 2010 HUDORA. All rights reserved.
"""


import datetime


def avg(data):
    """Calculate the average of a list.

    >>> avg([1,2,3])
    2.0
    >>> avg([1,1,10])
    4.0
    >>> avg([3])
    3.0
    """

    if data:
        return (sum(data)) / float(len(data))
    return 0.0


def median(data):
    """Calculated the Median of a list.

    >>> median([1,2,3])
    2.0
    >>> median([1,2,3,4])
    2.5
    >>> median([1,2,10])
    2.0
    >>> median([3])
    3.0
    """

    if data:
        sdata = sorted(data)
        if len(sdata) % 2 == 1:
            return float(sdata[(len(sdata) + 1) / 2 - 1])
        else:
            lower = sdata[len(sdata) / 2 - 1]
            upper = sdata[len(sdata) / 2]
            return float(lower + upper) / 2
    return 0.0


def _group_by_x(values, aggregationfunc, keyfunc, reversefunc):
    """input should be [(datetime, stuff), ...]"""

    groupings = {}
    for date, value in values:
        key = keyfunc(date)
        if key not in groupings:
            groupings[key] = [value]
        else:
            groupings[key].append(value)
    ret = []
    for key, values in groupings.items():
        ret.append((reversefunc(key), aggregationfunc(values)))
    ret.sort()
    return ret


def _month_key(date):
    return (date.year, date.month)


def _month_to_date(tup):
    """Reverse of _month_key()"""
    (year, month) = tup
    return datetime.date(year, month, 1)


def _quarter_key(date):
    return (date.year, int(((date.month + 1) / 3) + 1))


def _quarter_to_date(tup):
    """Reverse of _quarter_key()"""
    (year, month) = tup
    return datetime.date(year, month, 1)


def _year_key(date):
    return date.year


def _year_to_date(tup):
    """Reverse of _year_key()"""
    return datetime.date(tup, 1, 1)


def group_by_month(values, aggregationfunc):
    """input should be [(datetime, stuff), ...]"""
    return _group_by_x(values, aggregationfunc, _month_key, _month_to_date)


def group_by_quarter(values, aggregationfunc):
    """input should be [(datetime, stuff), ...]"""
    return _group_by_x(values, aggregationfunc, _quarter_key, _quarter_to_date)


def group_by_year(values, aggregationfunc):
    """input should be [(datetime, stuff), ...]"""
    return _group_by_x(values, aggregationfunc, _year_key, _year_to_date)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# encoding: utf-8
"""
hujson.py - extended json - tries to be compatible with simplejson

hujson can encode additional types like decimal and datetime into valid json.
All the heavy lifting is done by John Millikin's `jsonlib`, see
https://launchpad.net/jsonlib

Created by Maximillian Dornseif on 2010-09-10.
Copyright (c) 2010 HUDORA. All rights reserved.
"""


from _jsonlib import UnknownSerializerError
import _jsonlib
import datetime


def _unknown_handler(value):
    if isinstance(value, datetime.date):
        return str(value)
    elif isinstance(value, datetime.datetime):
        return value.isoformat() + 'Z'
    elif hasattr(value, 'properties'):
        return dict([(key, getattr(value, key)) for key in value.properties().keys()])
    elif 'google.appengine.api.users.User' in str(type(value)):
        return "%s/%s" % (value.user_id(), value.email())
    elif 'google.appengine.api.datastore_types.Key' in str(type(value)):
        return str(value)
    raise UnknownSerializerError("%s(%s)" % (type(value), value))


def dumps(val, indent=' '):
    return _jsonlib.write(val, on_unknown=_unknown_handler, indent=indent)


def loads(data):
    return _jsonlib.read(data)



#!/usr/bin/env python
# encoding: utf-8
"""
monetary.py -

Created by Christian Klein on 2010-03-25.
Copyright (c) 2010 Christian Klein. All rights reserved.
"""

import decimal


def cent_to_euro(amount):
    """
    Rechnet einen Cent-Betrag nach Euro um, inklusive Rundung

    Es wird mit ROUND_HALF_DOWN gerundet.

    >>> cent_to_euro(100)
    Decimal('1.00')
    >>> cent_to_euro(50)
    Decimal('0.50')
    >>> cent_to_euro(1)
    Decimal('0.01')
    >>> cent_to_euro('0.1')
    Decimal('0.00')
    """

    value = decimal.Decimal(amount) / 100
    return value.quantize(decimal.Decimal("0.01"), rounding=decimal.ROUND_HALF_DOWN)


def euro_to_cent(amount):
    """
    Rechnet einen Cent-Betrag nach Euro um, inklusive Rundung

    Es wird mit ROUND_HALF_DOWN gerundet.

    >>> euro_to_cent(1)
    Decimal('100')
    >>> euro_to_cent('0.5')
    Decimal('50')
    >>> euro_to_cent('0.01')
    Decimal('1')
    >>> euro_to_cent('0.001')
    Decimal('0')
    """

    value = decimal.Decimal(amount) * 100
    return value.quantize(decimal.Decimal("1"), rounding=decimal.ROUND_HALF_DOWN)


def netto(amount, tax=19):
    """
    Mehrwertsteuer aus den Preisen rausrechnen.

    >>> netto(decimal.Decimal('1.19'))
    Decimal('1.00')
    """

    amount /= decimal.Decimal("1.%d" % tax)
    return amount.quantize(decimal.Decimal("0.01"), rounding=decimal.ROUND_HALF_DOWN)


def brutto(amount, tax=19):
    """
    Mehrwertsteuer aus den Preisen reinrechnen.

    >>> brutto(decimal.Decimal('1.00'))
    Decimal('1.19')
    """

    amount *= decimal.Decimal("1.%d" % tax)
    return amount.quantize(decimal.Decimal("0.01"), rounding=decimal.ROUND_HALF_DOWN)


def tara(amount, tax=19):
    """
    Rechne die Differenz zwischen Brutto- und Nettopreis aus.

    >>> tara(decimal.Decimal('1.19'))
    Decimal('0.19')
    """
    return amount - netto(amount)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# encoding: utf-8
"""
daemon.py

Created by Maximillian Dornseif on 2009-08-26.
Copyright (c) 2009 HUDORA. All rights reserved.
"""


import os
import os.path
import sys


# Default daemon parameters.
# File mode creation mask of the daemon.
UMASK = 0

# Default working directory for the daemon.
WORKDIR = "/"

# Default maximum for the number of available file descriptors.
MAXFD = 1024

# The standard I/O file descriptors are redirected to /dev/null by default.
if (hasattr(os, "devnull")):
    REDIRECT_TO = os.devnull
else:
    REDIRECT_TO = "/dev/null"


def daemonize(pidfile=None, stdoutlogfile=None):
    """Detach a process from the controlling terminal and run it in the
    background as a daemon.

    based on http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/278731

    daemonize

    1. The current working directory set to the "/" directory.
    2. The current file creation mode mask set to 0.
    3. Close all open files (1024).
    4. Redirect standard I/O streams to "/dev/null".

    A failed call to fork() now raises an exception.

    References

    1. Advanced Programming in the Unix Environment: W. Richard Stevens
    2. Unix Programming Frequently Asked Questions:
            http://www.erlenstar.demon.co.uk/unix/faq_toc.html
    """

    try:
        # Fork a child process so the parent can exit. This returns control to
        # the command-line or shell. It also guarantees that the child will not
        # be a process group leader, since the child receives a new process ID
        # and inherits the parent's process group ID. This step is required
        # to ensure that the next call to os.setsid is successful.
        pid = os.fork()
    except OSError, e:
        raise RuntimeError("%s [%d]" % (e.strerror, e.errno))

    if (pid == 0):  # The first child.
        # To become the session leader of this new session and the process group
        # leader of the new process group, we call os.setsid(). The process is
        # also guaranteed not to have a controlling terminal.
        os.setsid()

        # Is ignoring SIGHUP necessary?
        #
        # It's often suggested that the SIGHUP signal should be ignored before
        # the second fork to avoid premature termination of the process. The
        # reason is that when the first child terminates, all processes, e.g.
        # the second child, in the orphaned group will be sent a SIGHUP.
        #
        # "However, as part of the session management system, there are exactly
        # two cases where SIGHUP is sent on the death of a process:
        #
        #   1) When the process that dies is the session leader of a session that
        #      is attached to a terminal device, SIGHUP is sent to all processes
        #      in the foreground process group of that terminal device.
        #   2) When the death of a process causes a process group to become
        #      orphaned, and one or more processes in the orphaned group are
        #      stopped, then SIGHUP and SIGCONT are sent to all members of the
        #      orphaned group." [2]
        #
        # The first case can be ignored since the child is guaranteed not to have
        # a controlling terminal.  The second case isn't so easy to dismiss.
        # The process group is orphaned when the first child terminates and
        # POSIX.1 requires that every STOPPED process in an orphaned process
        # group be sent a SIGHUP signal followed by a SIGCONT signal. Since the
        # second child is not STOPPED though, we can safely forego ignoring the
        # SIGHUP signal. In any case, there are no ill-effects if it is ignored.
        #
        # import signal # Set handlers for asynchronous events.
        # signal.signal(signal.SIGHUP, signal.SIG_IGN)

        try:
            # Fork a second child and exit immediately to prevent zombies.  This
            # causes the second child process to be orphaned, making the init
            # process responsible for its cleanup. And, since the first child is
            # a session leader without a controlling terminal, it's possible for
            # it to acquire one by opening a terminal in the future (System V-
            # based systems).  This second fork guarantees that the child is no
            # longer a session leader, preventing the daemon from ever acquiring
            # a controlling terminal.
            pid = os.fork()  # Fork a second child.
        except OSError, e:
            raise RuntimeError("%s [%d]" % (e.strerror, e.errno))

        if (pid == 0):  # The second child.
            # Since the current working directory may be a mounted filesystem, we
            # avoid the issue of not being able to unmount the filesystem at
            # shutdown time by changing it to the root directory.
            os.chdir(WORKDIR)
            # We probably don't want the file mode creation mask inherited from
            # the parent, so we give the child complete control over permissions.
            os.umask(UMASK)
        else:
            # exit() or _exit()?  See below.
            os._exit(0)  # Exit parent (the first child) of the second child.
    else:
        # exit() or _exit()?
        # _exit is like exit(), but it doesn't call any functions registered
        # with atexit (and on_exit) or any registered signal handlers. It also
        # closes any open file descriptors.  Using exit() may cause all stdio
        # streams to be flushed twice and any temporary files may be unexpectedly
        # removed. It's therefore recommended that child branches of a fork()
        # and the parent branch(es) of a daemon use _exit().
        os._exit(0)  # Exit parent of the first child.

    # write PID file
    pid = str(os.getpid())
    sys.stderr.write("\n%s started with PID %s" % (sys.argv[0], pid))
    sys.stderr.flush()
    if not pidfile:
        pidfile = '/var/run/%s.pid' % (sys.argv[0].split('/')[-1])
    if pidfile:
        file(pidfile, 'w+').write("%s\n" % pid)

    # Close all open file descriptors.  This prevents the child from keeping
    # open any file descriptors inherited from the parent.  There is a variety
    # of methods to accomplish this task.  Three are listed below.
    #
    # Try the system configuration variable, SC_OPEN_MAX, to obtain the maximum
    # number of open file descriptors to close.  If it doesn't exists, use
    # the default value (configurable).
    #
    # try:
    #    maxfd = os.sysconf("SC_OPEN_MAX")
    # except (AttributeError, ValueError):
    #    maxfd = MAXFD
    #
    # OR
    #
    # if (os.sysconf_names.has_key("SC_OPEN_MAX")):
    #    maxfd = os.sysconf("SC_OPEN_MAX")
    # else:
    #    maxfd = MAXFD
    #
    # OR
    #
    # Use the getrlimit method to retrieve the maximum file descriptor number
    # that can be opened by this process.  If there is not limit on the
    # resource, use the default value.
    #
    import resource  # Resource usage information.
    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
    if (maxfd == resource.RLIM_INFINITY):
        maxfd = MAXFD

    # Iterate through and close all file descriptors.
    for fd in range(0, maxfd):
        try:
            os.close(fd)
        except OSError:  # ERROR, fd wasn't open to begin with (ignored)
            pass

    # Redirect the standard I/O file descriptors to the specified file. Since
    # the daemon has no controlling terminal, most daemons redirect stdin,
    # stdout, and stderr to /dev/null. This is done to prevent side-effects
    # from reads and writes to the standard I/O file descriptors.

    if not stdoutlogfile:
        stdoutlogfile = '/var/log/%s.log' % (sys.argv[0].split('/')[-1])

    # This call to open is guaranteed to return the lowest file descriptor,
    # which will be 0 (stdin), since it was closed above.
    os.open(stdoutlogfile, os.O_RDWR | os.O_APPEND | os.O_CREAT)  # standard input (0)

    # Duplicate standard input to standard output and standard error.
    os.dup2(0, 1)  # standard output (1)
    os.dup2(0, 2)  # standard error (2)

    return 0



#!/usr/bin/env python
# encoding: utf-8

"""
pyJasper client.py - Way to talk to a pyJasper Server.
See http://pypi.python.org/pypi/pyJasper/ for further enligthenment.

Created by Maximillian Dornseif on 2007-10-12.
Moved into huTools in 2010, 2011
Consider it BSD licensed.
"""


from cStringIO import StringIO
from huTools.http import fetch
import os
import os.path
import re
import unittest
import urllib
import warnings
import xml.etree.ElementTree as ET


try:
    from django.conf import settings
    # trigger the lazy importer
    getattr(settings, 'DUMMY', None)
except (ImportError, EnvironmentError):
    settings = object()

config = object()
try:
    import config
except:
    pass


class JasperException(RuntimeError):
    """This exception indicates Jasper Server problem."""
    pass


def get_reportname(base, *args):
    """
    Construct path for report file relative to base

    In most cases, this will be JasperGenerator.__file__
    """
    warnings.warn("pyjasper.get_reportname is deprecated", DeprecationWarning, stacklevel=2)
    path = os.path.join(os.path.dirname(base), 'reports', *args)
    return os.path.abspath(path)


def _find_server(serverurl):
    if not serverurl:
        serverurl = getattr(settings, 'PYJASPER_SERVLET_URL', None)
    if not serverurl:
        serverurl = getattr(config, 'PYJASPER_SERVLET_URL', None)
    if not serverurl:
        serverurl = os.environ.get('PYJASPER_SERVLET_URL', None)
    if not serverurl:
        raise JasperException('set PYJASPER_SERVLET_URL')
    return serverurl


class JasperGenerator(object):
    """Abstract class for generating PDF Documents with Jasperreports.

    You have to overwrite generate_xml to make meaningfull use of this class. Then call
    YourClass.generate(yourdata). Yourdata is passed to generate_xml() and hopfully you will get
    the generated report back.
    """

    def __init__(self, serverurl=None, debug=False):
        super(JasperGenerator, self).__init__()
        self.reportname = None
        self.xpath = None
        self.debug = debug
        self.serverurl = _find_server(serverurl)
        warnings.warn("huTools.pyjasper.JasperGenerator() is deprecated use"
                      " huTools.pyjasper.generate_report() instead.", DeprecationWarning, stacklevel=2)

    def generate_xml(self, data=None):
        """To be overwritten by subclasses.

        E.g.
        def generate_xml(self, movement):
            xmlroot = ET.Element('report')
            ET.SubElement(xmlroot, 'generator').text = __revision__
            ET.SubElement(xmlroot, 'created_at').text = datetime.datetime.now().strftime("%Y-%m-%d %H:%M")
            xml_movement = ET.SubElement(xmlroot, 'movement')
            ET.SubElement(xml_movement, "location_from").text = unicode(movement.location_from)
            return xmlroot
        """
        raise NotImplementedError

    def get_xml(self, data=None):
        """Serializes the XML in the ElementTree to be send to JasperReports."""
        root = self.generate_xml(data)
        tree = ET.ElementTree(root)
        buf = StringIO()
        tree.write(buf, encoding="utf-8")
        ret = buf.getvalue()
        buf.close()
        return ret

    def get_report(self):
        """Get JasperReport template"""
        return open(self.reportname).read()

    def generate_pdf_server(self, design, xpath, xmldata, multi=False):
        """Generate report via pyJasperServer."""
        if multi:
            content = dict(designs=design, xpath=xpath, xmldata=xmldata)
        else:
            content = dict(design=design, xpath=xpath, xmldata=xmldata)
        status, _headers, content = fetch(self.serverurl, content, 'POST')

        if not status == 200:
            raise JasperException("%s -- %r" % (content, status))
        return content

    def generate_pdf(self, data=None):
        """Generates a PDF document by using Jasper-Reports."""
        design = self.get_report()
        xmldata = self.get_xml(data)
        if self.debug:
            open('/tmp/pyjasper-%s-debug.xml' % os.path.split(self.reportname)[-1], 'w').write(xmldata)
        return self.generate_pdf_server(design, self.xpath, xmldata)

    def generate(self, data=None):
        """Generates a report, returns the PDF."""
        return self.generate_pdf(data)


def generate_report(reportdesign, xpath, xmldata, url=None, sign_keyname='', sign_reason='',
                    callback='', metadata=None):
    """Generates a report, returns the PDF.

    reportdesign, xpath and xmldata - necessary data to generate a JasperReport.
    url - points to a jasper server
    sign_keyname - key for the signature lying on the server
    sign_reason - reason to be sent within the signed document
    callback - url to where the generated report will be sent
    metadata - PDF metadata

    Return value is pdf data stripped of timestamps for modification and creaton dates.
    If a callback is given, the server will sent the pdf data to the given URL instead.
    """
    url = _find_server(url)
    content = dict(design=reportdesign, xpath=xpath, xmldata=xmldata)
    if callback:
        content['callback'] = callback
    if sign_keyname:
        content['sign_keyname'] = sign_keyname
        if not sign_reason:
            raise ValueError('reason is needed when signing documents!')
        content['sign_reason'] = sign_reason
    if metadata:
        content['metadata'] = urllib.urlencode(metadata)

    status, _headers, content = fetch(url, content, 'POST')
    if not status == 200:
        raise JasperException("%s -- %r" % (content, status))
    # remove Timestamps so the same reports hash the same.
    return re.sub("/(ModDate|CreationDate)\(D:\d{14}[+-]\d{2}'\d{2}'\)", '', content)


_testreport = """<?xml version="1.0" encoding="UTF-8"?>
        <jasperReport xmlns="http://jasperreports.sourceforge.net/jasperreports"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
          xsi:schemaLocation="http://jasperreports.sourceforge.net/jasperreports
 http://jasperreports.sourceforge.net/xsd/jasperreport.xsd"
          name="Lieferschein" pageWidth="595" pageHeight="842" columnWidth="483" leftMargin="56"
          rightMargin="56" topMargin="70" bottomMargin="28" whenResourceMissingType="Key">
            <queryString language="xPath"><![CDATA[/elements/element]]></queryString>
            <field name="testdata" class="java.lang.String">
                <fieldDescription><![CDATA[*/data]]></fieldDescription>
            </field>
            <detail>
                <band height="32">
                    <staticText>
                        <reportElement x="20" y="20" width="200" height="10"/>
                        <text><![CDATA[Hello World!]]></text>
                    </staticText>
                    <textField isBlankWhenNull="true">
                        <reportElement key="field-1" x="36" y="0" width="300" height="10"/>
                        <textElement><font size="8"/></textElement>
                <textFieldExpression class="java.lang.String"><![CDATA[$F{testdata}]]></textFieldExpression>
                    </textField>
                </band>
            </detail>
        </jasperReport>
        """


class _TestGenerator(JasperGenerator):

    def generate_xml(self, dummy):
        self.xpath = '/elements/element'
        xmlroot = ET.Element('elements')
        xml_movement = ET.SubElement(xmlroot, 'element')
        ET.SubElement(xml_movement, "data").text = unicode('TESTTEXT')
        return xmlroot

    def get_report(self):
        # unfortunately this example data always generates an empty report. Why?
        # it's at least enough for testing
        self.reportname = 'Nada'
        return _testreport


class testTests(unittest.TestCase):

    def test_legacy_class(self):
        gen = _TestGenerator()
        gen.generate()

    def test_function(self):
        generate_report(_testreport,
                        '/elements/element',
                        '<elements><element><data>TEST</data></element></elements>',
                        sign_keyname="hudora-rechnungen", sign_reason='Testreason for generating documents')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
test.py

Created by Maximillian Dornseif on 2010-10-24.
Copyright (c) 2010 HUDORA. All rights reserved.
"""

import unittest
from huTools.http import fetch


class testTests(unittest.TestCase):

    def test_fetch_get_basic(self):
        status, header, body = fetch('http://www.google.com')
        assert status == 200
        status, header, body = fetch('http://www.postbin.org/186ndf2', {'q': 'hudora'})
        assert status == 200
        status, header, body = fetch('http://www.postbin.org/186ndf2', {'keyg': 'value', 'just a test': 212})
        assert status == 200

    def test_fetch_get_unicode(self):
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'kühg1': 'Iñtërnâtiônàlizætiøn'})
        assert status == 200
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'kühg2': u'Iñtërnâtiônàlizætiøn'})
        assert status == 200
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {u'kühg3': 'Iñtërnâtiônàlizætiøn'})
        assert status == 200

    def test_fetch_post_basic(self):
        status, header, body = fetch('http://www.postbin.org/186ndf2',
        {'keyp1': 'value', 'just a test': 212}, 'POST')
        assert status == 201

    def test_fetch_post_file(self):
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'hosts': open('/etc/hosts', 'r')}, 'POST')
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'hosts': open('/etc/hosts', 'r'),
                                      'unicode': u'Iñtërnâtiônàlizætiøn'}, 'POST')
        assert status == 201

    def test_fetch_post_unicode(self):
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'kuehp1': 'Iñtërnâtiônàlizætiøn'}, 'POST')
        assert status == 201
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'kühp2': 'Iñtërnâtiônàlizætiøn'}, 'POST')
        assert status == 201
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {'kühp3': u'Iñtërnâtiônàlizætiøn'}, 'POST')
        assert status == 201
        status, header, body = fetch('http://www.postbin.org/186ndf2',
                                     {u'kühp4': u'Iñtërnâtiônàlizætiøn'}, 'POST')
        assert status == 201


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
bank.py

Created by Christian Klein on 2010-03-25.
Copyright (c) 2010 Christian Klein. All rights reserved.
"""

from string import ascii_uppercase


def convert_character(string):
    """Konvertierung von nicht-numerischen Zeichen in einer IBAN"""
    result = []
    for character in string:
        if character.isdigit():
            result.append(character)
        else:
            result.append(str(ascii_uppercase.index(character) + 10))

    return "".join(result)


def iban(ktonr, blz, land="DE"):
    """
    Berechnet die International Bank Account Number für deutsche Konten

    Zunächst wird die Basic Bank Account Number für deutsche Konten berechnet.
    Der Laendercode wird mit zwei '0' aufgefüllt und an die BBAN gehangen.
    Aus diesem wird der Prüfcode als Modulo 97-10 (DIN ISO 7064) berechnet.

    >>>
    """

    bban = "%08d%010d" % (int(blz), int(ktonr))
    tmp = convert_character("%s%s00" % (bban, land))
    pruefziffer = 98 - (int(tmp) % 97)
    return "%s%02d%s" % (land, pruefziffer, bban)


def check_iban(value):
    """Validiere (errechnete) IBAN"""
    return int(convert_character(value[4:] + value[:4])) % 97 == 1



#!/usr/bin/env python
# encoding: utf-8
"""
engine_appengine.py implements urlfetch based queries for huTools.http.

See http://code.google.com/appengine/docs/python/urlfetch/overview.html for the basics of the
AppEngine capabilities.

Created by Maximillian Dornseif on 2010-10-24.
Copyright (c) 2010 HUDORA. All rights reserved.
"""


from google.appengine.api import urlfetch
import logging


def request(url, method, content, headers, timeout=25):
    """Does a HTTP Request via Google Appengine urlfetch Service.

    Incereases the default appengine timeout from 5 seconds to 25.
    (For Web-Requests this is reduced to 10s by the GAE Infrastructure)"""

    # on appengine debuging is always post mortem, so better log what we arde doing
    logging.debug('fetching %s %s', method, url)
    if method == 'GET':
        method = urlfetch.GET
    elif method == 'POST':
        method = urlfetch.POST
    elif method == 'DELETE':
        method = urlfetch.DELETE
    elif method == 'PUT':
        method = urlfetch.PUT
    elif method == 'HEAD':
        method = urlfetch.HEAD
    headers['User-Agent'] = headers.get('User-Agent', '') + ' (urlfetch)'
    result = urlfetch.fetch(url=url, deadline=timeout, payload=content, method=method, headers=headers)
    return int(result.status_code), result.headers, result.content



#!/usr/bin/env python
# encoding: utf-8

"""
Copyright (c) 2007 HUDORA GmbH. BSD Licensed.
"""

import doctest
import sys
import unicodedata
from types import StringType

__revision__ = "$Revision$"


def deUTF8(data):
    """This is meant to help with utf-8 data appearing where unicode should apperar."""
    # In particular my DB is returning the wrong thing.
    if isinstance(data, StringType):
        return data.decode('utf-8')
    return data


# native, HTML, default Unicode (Code page 850), Unicode combined Character, Windows-1250
_recodings = {'ae': ['ä', u'ä', '&auml;', '\u00E4', u'\u00E4', '\u0308a', '\xc3\xa4'],
              'oe': ['ö', u'ö', '&ouml;', '\u00F6', u'\u00F6', '\u0308o', '\xc3\xb6', u'á'],
              'ue': ['ü', u'ü', '&uuml;', '\u00FC', u'\u00FC', '\u0308u', '\xc3\xbc'],
              'Ae': ['Ä', u'Ä', '&Auml;', '\u00C4', u'\u00C4', '\u0308A', '\xc3\x84'],
              'Oe': ['Ö', u'Ö', '&Ouml;', '\u00D6', u'\u00D6', '\u0308O', '\xc3\x96', u'Ó'],
              'Ue': ['Ü', u'Ü', '&Uuml;', '\u00DC', u'\u00DC', '\u0308U', '\xc3\x9c'],
              'ss': ['ß', u'ß', '&szlig;', '\u00DF', u'\u00DF', '\xc3\x9f', u'ß'],
              'e': ['é', u'é', '\xc3\xa9', u'ê', u'è'],
              'i': [u'í', u'í'],
              'E': [u'É', u'È'],
              "'": [u'´', '´', u'`', '`'],
             }


def deUmlaut(data):
    """Converts a text to ASCII acting smart about Umlauts.

    >>> deUmlaut('1 Über Hügel saß René äöüÄÖÜß')
    '1 Ueber Huegel sass Rene aeoeueAeOeUess'
    """

    for to_char, from_chars in _recodings.items():
        for from_char in from_chars:
            try:
                data = data.replace(from_char, to_char)
            except UnicodeDecodeError:
                data = data

    data = unicodedata.normalize('NFKD', data)

    try:
        return data.encode('ascii', 'replace')
    except UnicodeEncodeError, msg:
        raise ValueError('%s: %r' % (msg, data))
    except UnicodeDecodeError, msg:
        raise ValueError('%s: %r' % (msg, data))


if __name__ == '__main__':
    failure_count, test_count = doctest.testmod()
    sys.exit(failure_count)



#!/usr/bin/env python
# encoding: utf-8
"""
engine_httplib2.py implements httplib2 based queries for huTools.http

Created by Maximillian Dornseif on 2010-10-24.
Copyright (c) 2010 HUDORA. All rights reserved.
"""


_http = None


def request(url, method, content, headers, timeout=15):
    """Does a HTTP Request via httplib2 Service."""

    global _http
    if not _http:
        import huTools.http._httplib2
        _http = huTools.http._httplib2.Http()

    headers['User-Agent'] = headers.get('User-Agent', '') + ' (httplib2)'
    resp, content = _http.request(url, method, content, headers=headers)
    replyheaders = {}
    replyheaders.update(resp)
    return int(resp.status), replyheaders, content



#!/usr/bin/env python
# encoding: utf-8
"""
printing.py simple minded toolkit for printing on CUPS.

Created by Maximillian Dornseif on 2006-11-19. BSD Licensed.
"""

import os
import os.path
import subprocess


__revision__ = "$Revision$"


def print_file(filename, jobname=None, printer=None, copies=1):
    """Print a file."""

    if not os.path.exists(filename):
        return

    args = ['/usr/local/bin/lpr', '-#%d' % copies]
    if printer:
        args.append('-P%s' % str(printer))
    args.append('"%s"' % filename)
    subprocess.call(args)


def print_data(data, jobname=None, printer=None, copies=1, printserver='printserver.local.hudora.biz'):
    """Print a datastream."""
    args = ['/usr/local/bin/lpr', '-#%d' % copies]
    if printer:
        args.append('-P%s' % str(printer))

    #if printserver:
    #    args.append('-H %s' % printserver)
    if jobname:
        args.append('-J %s  ' % jobname.replace("'\";./ ", "_"))

    pipe = subprocess.Popen(args, shell=False, stdin=subprocess.PIPE).stdin
    pipe.write(data)
    pipe.close()
